"""
validates parsing
"""
import re
from typing import Optional, Tuple

from nepali.char import nepali_to_english_text
from nepali.constants import NEPALI_MONTHS_EN, WEEKS_ABBR_EN, WEEKS_EN
from nepali.datetime import nepalidatetime, nepalimonth, nepaliweek

__nepali_time_re__CACHE = None


class NepaliTimeRE(dict):
    def __init__(self):
        """Create keys/values.
        Order of execution is important for dependency reasons.
        """
        base = super()
        base.__init__(
            {
                # The " [1-9]" part of the regex is to make %c from ANSI C work
                "d": r"(?P<d>3[0-2]|[1-2]\d|0[1-9]|[1-9]| [1-9])",
                "-d": r"(?P<d>3[0-2]|[1-2]\d|0[1-9]|[1-9]| [1-9])",  # same as "d"
                "f": r"(?P<f>[0-9]{1,6})",
                "H": r"(?P<H>2[0-3]|[0-1]\d|\d)",
                "-H": r"(?P<H>2[0-3]|[0-1]\d|\d)",
                "I": r"(?P<I>1[0-2]|0[1-9]|[1-9])",
                "-I": r"(?P<I>1[0-2]|0[1-9]|[1-9])",
                "G": r"(?P<G>\d\d\d\d)",
                "j": r"(?P<j>36[0-6]|3[0-5]\d|[1-2]\d\d|0[1-9]\d|00[1-9]|[1-9]\d|0[1-9]|[1-9])",
                "m": r"(?P<m>1[0-2]|0[1-9]|[1-9])",
                "-m": r"(?P<m>1[0-2]|0[1-9]|[1-9])",  # same as "m"
                "M": r"(?P<M>[0-5]\d|\d)",
                "-M": r"(?P<M>[0-5]\d|\d)",  # same as "M"
                "S": r"(?P<S>6[0-1]|[0-5]\d|\d)",
                "-S": r"(?P<S>6[0-1]|[0-5]\d|\d)",  # same as "S"
                "w": r"(?P<w>[0-6])",
                "y": r"(?P<y>\d\d)",
                "Y": r"(?P<Y>\d\d\d\d)",
                "z": r"(?P<z>[+-]\d\d:?[0-5]\d(:?[0-5]\d(\.\d{1,6})?)?|(?-i:Z))",
                "A": self.__seqToRE(WEEKS_EN, "A"),
                "a": self.__seqToRE(WEEKS_ABBR_EN, "a"),
                "B": self.__seqToRE(NEPALI_MONTHS_EN, "B"),
                "b": self.__seqToRE(NEPALI_MONTHS_EN, "b"),
                "p": self.__seqToRE(
                    (
                        "AM",
                        "PM",
                    ),
                    "p",
                ),
                "%": "%",
            }
        )

    def __seqToRE(self, to_convert, directive):
        """Convert a list to a regex string for matching a directive.
        Want possible matching values to be from longest to shortest.  This
        prevents the possibility of a match occurring for a value that also
        a substring of a larger value that should have matched (e.g., 'abc'
        matching when 'abcdef' should have been the match).
        """
        to_convert = sorted(to_convert, key=len, reverse=True)
        for value in to_convert:
            if value != "":
                break
        else:
            return ""
        regex = "|".join(re.escape(stuff) for stuff in to_convert)
        regex = f"(?P<{directive}>{regex}"
        return f"{regex})"

    def pattern(self, date_format):
        """
        Handle conversion from format directives to regex.
        """
        processed_format = ""
        regex_chars = re.compile(r"([\\.^$*+?\(\){}\[\]|])")
        date_format = regex_chars.sub(r"\\\1", date_format)
        whitespace_replacement = re.compile(r"\s+")
        date_format = whitespace_replacement.sub(r"\\s+", date_format)
        while "%" in date_format:
            directive_index = date_format.index("%") + 1
            index_increment = 1

            if date_format[directive_index] == "-":
                index_increment = 2

            if (
                date_format[directive_index : directive_index + index_increment]
                not in self
            ):
                return None

            processed_format = "%s%s%s" % (
                processed_format,
                date_format[: directive_index - 1],
                self[date_format[directive_index : directive_index + index_increment]],
            )
            date_format = date_format[directive_index + index_increment :]
        return f"^{processed_format}{date_format}$"

    def compile(self, date_format):
        """Return a compiled re object for the format string."""
        return re.compile(self.pattern(date_format), re.IGNORECASE)


def get_nepali_time_re_object():
    global __nepali_time_re__CACHE
    if __nepali_time_re__CACHE is None:
        __nepali_time_re__CACHE = NepaliTimeRE()
    return __nepali_time_re__CACHE


def extract(datetime_str, format):
    """
    Extracts year, month, day, hour, minute, etc from the given format.

    eg.
    USAGE: extract("2078-01-12", format="%Y-%m-%d")
    INPUT:
    datetime_str="2078-01-12"
    format="%Y-%m-%d"

    OUTPUT:
    {
        "Y": 2078,
        "m": 1,
        "d": 12,
    }
    """

    # converting datetime_str to english if any exists
    datetime_str = nepali_to_english_text(datetime_str)

    re_compiled_format = get_nepali_time_re_object().compile(format)
    match = re_compiled_format.match(datetime_str)
    if match is None:
        return {}
    return match.groupdict()


def __convert_12_hour_to_24_hour(hour: int, am_pm: str) -> int:
    """Converts hours from 12-hour format to 24-hour format.

    :param hour: The hour value to convert.
    :param am_pm: Either "am" or "pm"; signifies whether the hour is in am or pm.

    :returns: The value of `hour` converted to 24-hour format.
    """
    am_pm = am_pm.lower()
    if am_pm == "am" and hour == 12:
        return 0
    elif am_pm == "pm" and hour != 12:
        return hour + 12
    return hour


def __calculate_year(data: dict) -> Optional[int]:
    """Calculates the year value from given data.

    :param data: The dictionary of the format:
                    {
                        "Y": 2078,
                        "b": "Mangsir",
                        "d": 12,
                        ...
                    }

    :returns: The year value of given date data.
    """
    if "y" in data:
        return int(data["y"]) + 2000
    elif "Y" in data:
        return int(data["Y"])
    return None


def __calculate_month(data: dict) -> nepalimonth:
    """Calculates the month value from given data.

    :param data: The dictionary of the format:
                    {
                        "Y": 2078,
                        "b": "Mangsir",
                        "d": 12,
                        ...
                    }

    :returns: The month value of given date data.
    """
    if "m" in data:
        return nepalimonth(int(data["m"]))
    elif "b" in data:
        return nepalimonth(data["b"])
    elif "B" in data:
        return nepalimonth(data["B"])
    return nepalimonth(1)


def __calculate_day(data: dict) -> int:
    """Calculates the day value from given data.

    :param data: The dictionary of the format:
                    {
                        "Y": 2078,
                        "b": "Mangsir",
                        "d": 12,
                        ...
                    }

    :returns: The day value of given date data.
    """
    if "d" in data:
        return int(data["d"])
    return 1


def __calculate_hour_minute_seconds(data: dict) -> Tuple[int, int, int, int]:
    """Calculates hour, minutes, seconds and microseconds from given data.

    :param data: The dictionary of the format:
                    {
                        "Y": 2078,
                        "b": "Mangsir",
                        "d": 12,
                        "H": 12,
                        "M": 12,
                        "S": 12,
                        "f": 12,
                        ...
                    }

    :returns: A tuple of hour, minute, seconds and microseconds.
    """
    hour = minute = second = fraction = 0
    if "H" in data:
        hour = int(data["H"])
    elif "I" in data:
        am_pm = data.get("p", "").lower() or "am"
        hour = __convert_12_hour_to_24_hour(hour=int(data["I"]), am_pm=am_pm)

    if "M" in data:
        minute = int(data["M"])

    if "S" in data:
        second = int(data["S"])

    if "f" in data:
        s = data["f"]
        # Pad to always return microseconds.
        s += "0" * (6 - len(s))
        fraction = int(s)

    return hour, minute, second, fraction


def __calculate_weekday(data: dict) -> Optional[nepaliweek]:
    """Calculates the weekday of the date given in data.

    :param data: The data that describes the date.

    :returns: The weekday value; 0 for Sunday, 1 for Monday, etc.
    """
    if "a" in data:
        return nepaliweek(data["a"])
    elif "A" in data:
        return nepaliweek(data["A"])
    elif "w" in data:
        return nepaliweek((int(data["w"]) - 1) % 7)
    return None


def transform(data: dict):
    """
    transforms different format data to uniform data

    eg.
    INPUT:
    data = {
        "Y": 2078,
        "b": "Mangsir",
        "d": 12,
        ...
    }

    OUTPUT:
    {
        "year": 2078,
        "month": 8,
        "day": 12,
        ...
    }
    """

    year = __calculate_year(data)
    month = __calculate_month(data)
    day = __calculate_day(data)
    hour, minute, second, fraction = __calculate_hour_minute_seconds(data)
    weekday = __calculate_weekday(data)

    return {
        "year": year,
        "month": month,
        "day": day,
        "hour": hour,
        "minute": minute,
        "second": second,
        "microsecond": fraction,
        "weekday": weekday,
    }


def validate(datetime_str, format):
    """
    validates datetime_str with the format
    Perform step by step test for fast performance. The steps are:
    -
    -
    returns False if validation failed
    returns nepalidatetime object if validation success.
    """

    # 1. validate if format is correct.
    if get_nepali_time_re_object().pattern(format) is None:
        # regex pattern generation failed
        return None

    # 2. validate if parse result if not empty
    parsed_result = extract(datetime_str, format)
    if parsed_result.get("Y") is None and parsed_result.get("y") is None:
        # compilation failed or year included
        return None

    # 3. validate if transformation
    transformed_data = transform(parsed_result)
    if transformed_data.get("year") is None:
        # could not transform data, not getting year
        return None

    # 4. create the datetime object
    return nepalidatetime(
        year=transformed_data["year"],
        month=transformed_data["month"],
        day=transformed_data["day"],
        hour=transformed_data["hour"],
        minute=transformed_data["minute"],
        second=transformed_data["second"],
        microsecond=transformed_data["microsecond"],
    )



from functools import cached_property
from typing import Any, Dict, Optional, Union

from nepali.constants import WEEKS_ABBR_EN, WEEKS_ABBR_NE, WEEKS_EN, WEEKS_NE


class NepaliWeekMeta(type):
    _cache: Dict[int, "nepaliweek"] = {}

    def __call__(cls, week: Union[int, str], *args, **kwargs) -> "nepaliweek":
        """
        Parses the week data and manages the cache.

        :param week: An integer or string representing the week.
        :type week: Union[int, str]
        :return: An instance of the nepaliweek class representing the given week.
        :rtype: nepaliweek
        :raises ValueError: If the given week is invalid.
        """
        value: Optional[int] = None

        if isinstance(week, int):
            value = int(week)
        elif isinstance(week, str):
            try:
                value = cls._parse_str(week)
            except ValueError:
                pass

        # checking if week is valid
        if value is None or not 0 <= value <= 6:
            raise ValueError(f"Invalid week: {week}")

        # checking cache
        if value not in cls._cache:
            cls._cache[value] = super().__call__(value, *args, **kwargs)

        return cls._cache[value]

    @staticmethod
    def _parse_str(week: str) -> int:
        """
        Parses str value of the week and returns int.

        :param week: A string representing the week.
        :type week: str
        :return: An integer representing the week.
        :rtype: int
        :raises ValueError: If the given string does not represent a valid week.
        """
        if week.isdigit():
            return int(week)

        week = week.capitalize()
        week_names = WEEKS_EN + WEEKS_NE + WEEKS_ABBR_EN + WEEKS_ABBR_NE
        try:
            index = week_names.index(week)
        except ValueError:
            raise ValueError(f"Invalid week name: {week}")

        return index % 7


class nepaliweek(metaclass=NepaliWeekMeta):
    """
    Represents Nepali week: Sunday, Monday, ..., Saturday.
    Sunday: 0,
    Monday: 1,
    ...
    Saturday: 6

    >>> nepaliweek(0)
    >>> nepaliweek("Sunday")
    >>> nepaliweek("आइतबार")
    >>> nepaliweek("Sun")

    :param Union[int, str] week: Week data to be parsed.
    :raises ValueError: The value is invalid.
    """

    def __init__(self, week) -> None:
        self.__value = week

    def __str__(self) -> str:
        return self.name

    def __repr__(self) -> str:
        return f"<nepaliweek: {self.__value}>"

    def __int__(self) -> int:
        return self.value

    def __eq__(self, other: Any) -> bool:
        if isinstance(other, nepaliweek):
            return self.value == other.value
        elif isinstance(other, int):
            return self.value == other

        return False

    @cached_property
    def value(self) -> int:
        return self.__value

    @cached_property
    def name(self) -> str:
        """Week's english name"""
        return WEEKS_EN[self.__value]

    @cached_property
    def abbr(self) -> str:
        """Week's english abbreviated name"""
        return WEEKS_ABBR_EN[self.__value]

    @cached_property
    def name_ne(self) -> str:
        """Week's nepali name"""
        return WEEKS_NE[self.__value]

    @cached_property
    def abbr_ne(self) -> str:
        """Week's nepali abbreviated name"""
        return WEEKS_ABBR_NE[self.__value]



"""
Exceptions for nepali
"""


class InvalidDateFormatException(Exception):
    pass


class InvalidDateTimeFormatException(Exception):
    pass


class InvalidNepaliDateTimeObjectException(Exception):
    pass


class FormatNotMatchException(Exception):
    """
    raised while parsing nepalidatetime format
    """

    pass



"""
To run only this unit test use the command below.

    python -m unittest nepali/tests/test_nepalimonth.py -v
"""
import unittest

from nepali.datetime import nepalimonth


class TestNepaliMonth(unittest.TestCase):
    # parse
    def test_nepalimonth_parses_int(self):
        self.assertEqual(nepalimonth(1).value, 1)
        self.assertEqual(nepalimonth(12).value, 12)

    def test_nepalimonth_fails_for_int_less_than_1(self):
        with self.assertRaises(ValueError) as ex:
            nepalimonth(0)
        self.assertEqual(str(ex.exception), "Invalid month: 0")
        with self.assertRaises(ValueError):
            nepalimonth(-100)

    def test_nepalimonth_fails_for_int_more_than_12(self):
        with self.assertRaises(ValueError) as ex:
            nepalimonth(13)
        self.assertEqual(str(ex.exception), "Invalid month: 13")
        with self.assertRaises(ValueError):
            nepalimonth(100)

    def test_nepalimonth_parses_str_int_number(self):
        month = nepalimonth("1")
        self.assertEqual(month.value, 1)

    def test_nepalimonth_fails_for_str_int_out_of_range(self):
        with self.assertRaises(ValueError):
            nepalimonth("0")
        with self.assertRaises(ValueError):
            nepalimonth("-100")
        with self.assertRaises(ValueError):
            nepalimonth("13")
        with self.assertRaises(ValueError):
            nepalimonth("100")

    def test_nepalimonth_fails_for_invalid_str(self):
        with self.assertRaises(ValueError):
            nepalimonth("hello")
        with self.assertRaises(ValueError):
            nepalimonth("1.0")

    def test_nepalimonth_parse_valid_month_in_english(self):
        self.assertEqual(nepalimonth("Baishakh").value, 1)
        self.assertEqual(nepalimonth("Jestha").value, 2)
        self.assertEqual(nepalimonth("Ashad").value, 3)
        self.assertEqual(nepalimonth("Sharwan").value, 4)
        self.assertEqual(nepalimonth("Bhadra").value, 5)
        self.assertEqual(nepalimonth("Ashwin").value, 6)
        self.assertEqual(nepalimonth("Kartik").value, 7)
        self.assertEqual(nepalimonth("Mangsir").value, 8)
        self.assertEqual(nepalimonth("Poush").value, 9)
        self.assertEqual(nepalimonth("Magh").value, 10)
        self.assertEqual(nepalimonth("Falgun").value, 11)
        self.assertEqual(nepalimonth("Chaitra").value, 12)

    def test_nepalimonth_parse_valid_month_in_english_lower_case(self):
        self.assertEqual(nepalimonth("baishakh").value, 1)
        self.assertEqual(nepalimonth("jestha").value, 2)
        self.assertEqual(nepalimonth("ashad").value, 3)
        self.assertEqual(nepalimonth("sharwan").value, 4)
        self.assertEqual(nepalimonth("bhadra").value, 5)
        self.assertEqual(nepalimonth("ashwin").value, 6)
        self.assertEqual(nepalimonth("kartik").value, 7)
        self.assertEqual(nepalimonth("mangsir").value, 8)
        self.assertEqual(nepalimonth("poush").value, 9)
        self.assertEqual(nepalimonth("magh").value, 10)
        self.assertEqual(nepalimonth("falgun").value, 11)
        self.assertEqual(nepalimonth("chaitra").value, 12)

    def test_nepalimonth_parse_valid_month_in_english_case_insensitive(self):
        self.assertEqual(nepalimonth("bAisHakh").value, 1)
        self.assertEqual(nepalimonth("jEsTha").value, 2)
        self.assertEqual(nepalimonth("aShaD").value, 3)
        self.assertEqual(nepalimonth("shaRwaN").value, 4)
        self.assertEqual(nepalimonth("BhadRa").value, 5)
        self.assertEqual(nepalimonth("ashWIn").value, 6)
        self.assertEqual(nepalimonth("KArtIk").value, 7)
        self.assertEqual(nepalimonth("MaNGsir").value, 8)
        self.assertEqual(nepalimonth("POush").value, 9)
        self.assertEqual(nepalimonth("maGh").value, 10)
        self.assertEqual(nepalimonth("faLgUn").value, 11)
        self.assertEqual(nepalimonth("ChaItRa").value, 12)

    def test_nepalimonth_parse_valid_month_in_nepali(self):
        self.assertEqual(nepalimonth("बैशाख").value, 1)
        self.assertEqual(nepalimonth("जेठ").value, 2)
        self.assertEqual(nepalimonth("असार").value, 3)
        self.assertEqual(nepalimonth("साउन").value, 4)
        self.assertEqual(nepalimonth("भदौ").value, 5)
        self.assertEqual(nepalimonth("असोज").value, 6)
        self.assertEqual(nepalimonth("कात्तिक").value, 7)
        self.assertEqual(nepalimonth("मंसिर").value, 8)
        self.assertEqual(nepalimonth("पुस").value, 9)
        self.assertEqual(nepalimonth("माघ").value, 10)
        self.assertEqual(nepalimonth("फागुन").value, 11)
        self.assertEqual(nepalimonth("चैत").value, 12)

    def test_nepalimonth_fails_for_invalid_data_type(self):
        with self.assertRaises(ValueError):
            nepalimonth(1.0)  # type: ignore
        with self.assertRaises(ValueError):
            nepalimonth(2j + 1)  # type: ignore
        with self.assertRaises(ValueError):
            nepalimonth([])  # type: ignore

    def test_nepalimonth_str(self):
        self.assertEqual(str(nepalimonth(1)), nepalimonth(1).name)

    def test_nepalimonth_repr(self):
        self.assertEqual(repr(nepalimonth(1)), "<nepalimonth: 1>")

    def test_nepalimonth_name(self):
        self.assertEqual(nepalimonth(1).name, "Baishakh")
        self.assertEqual(nepalimonth(12).name, "Chaitra")

    def test_nepalimonth_name_ne(self):
        self.assertEqual(nepalimonth(1).name_ne, "बैशाख")
        self.assertEqual(nepalimonth(12).name_ne, "चैत")

    def test_nepalimonth_variable_cache(self):
        self.assertEqual(id(nepalimonth(1)), id(nepalimonth(1)))
        self.assertEqual(id(nepalimonth(12)), id(nepalimonth(12)))

    def test_nepalimonth_int(self):
        self.assertEqual(int(nepalimonth(1)), 1)
        self.assertEqual(int(nepalimonth(12)), 12)

    def test_nepalimonth_equal_with_nepalimonth(self):
        self.assertEqual(nepalimonth(1), nepalimonth(1))
        self.assertEqual(nepalimonth(12), nepalimonth(12))

    def test_nepalimonth_not_equal(self):
        self.assertNotEqual(nepalimonth(1), nepalimonth(2))
        self.assertNotEqual(nepalimonth(1), 2)

    def test_nepalimonth_equal_with_int(self):
        self.assertEqual(nepalimonth(1), 1)
        self.assertEqual(nepalimonth(12), 12)
        self.assertEqual(1, nepalimonth(1))
        self.assertEqual(12, nepalimonth(12))

    def test_nepalimonth_not_equal_with_invalid_object(self):
        self.assertNotEqual(nepalimonth(1), "invalid")



"""
To run only this unit test use the command below.

    python -m unittest nepali/tests/test_timezone.py -v
"""
import datetime
import unittest
from unittest.mock import patch

from nepali.constants import NEPAL_TIMEZONE
from nepali.timezone import (
    NepaliTimeZone,
    get_timezone,
    now,
    to_nepali_timezone,
    to_utc_timezone,
    utc_now,
)


class TestNepaliTimeZone(unittest.TestCase):
    def setUp(self):
        self.nepali_timezone = NepaliTimeZone()
        return super().setUp()

    def test_nepali_timezone_dst(self):
        self.assertEqual(self.nepali_timezone.dst(None), datetime.timedelta(0))

    def test_nepali_timezone_utcoffset(self):
        self.assertEqual(
            self.nepali_timezone.utcoffset(None),
            datetime.timedelta(hours=5, minutes=45),
        )

    def test_nepali_timezone_tzname(self):
        self.assertEqual(self.nepali_timezone.tzname(None), NEPAL_TIMEZONE)

    def test_nepali_timezone_str(self):
        self.assertEqual(str(self.nepali_timezone), NEPAL_TIMEZONE)

    def test_nepali_timezone_repr(self):
        self.assertEqual(repr(self.nepali_timezone), NEPAL_TIMEZONE)

    def test_datetime_with_nepali_timezone(self):
        dt = datetime.datetime(2015, 10, 14, tzinfo=self.nepali_timezone)
        self.assertEqual(dt.tzinfo, self.nepali_timezone)
        self.assertEqual(dt.tzname(), self.nepali_timezone.tzname(dt))
        self.assertEqual(dt.utcoffset(), self.nepali_timezone.utcoffset(dt))


class TestTimezoneUtils(unittest.TestCase):
    def test_get_timezone(self):
        tz = get_timezone()
        self.assertIsNotNone(tz)

    @patch("nepali.timezone.get_timezone")
    @patch("nepali.timezone.datetime")
    def test_timezone_now(self, mock_datetime, mock_get_timezone):
        # mocking data
        mock_get_timezone.return_value = (
            datetime.timezone.utc
        )  # mocking get_timezone => UTC
        datetime_now = mock_datetime.datetime.now
        datetime_now.return_value = datetime.datetime(
            2015, 1, 1
        )  # mocking datetime.now => 2015/1/1

        # calling now
        current_dt = now()

        # checking
        self.assertEqual(current_dt, datetime.datetime(2015, 1, 1))
        self.assertSequenceEqual(datetime_now.call_args[0], [datetime.timezone.utc])

    @patch("nepali.timezone.datetime")
    def test_timezone_utc_now(self, mock_datetime):
        # mocking data
        mock_datetime.timezone.utc = (
            datetime.timezone.utc
        )  # reverting the utc timezone mock
        datetime_now = mock_datetime.datetime.now
        datetime_now.return_value = datetime.datetime(
            2015, 1, 1
        )  # mocking datetime.now => 2015/1/1

        # calling utc_now
        utc_dt = utc_now()

        # checking
        self.assertEqual(utc_dt, datetime.datetime(2015, 1, 1))
        self.assertSequenceEqual(datetime_now.call_args[0], [datetime.timezone.utc])

    # to_utc_timezone
    def test_to_utc_timezone_invalid_input_returns_as_it_is(self):
        self.assertEqual(to_utc_timezone("test"), "test")  # type: ignore

    @patch("nepali.timezone.get_timezone")
    def test_to_utc_timezone_without_timezone(self, mock_get_timezone):
        mock_get_timezone.return_value = datetime.timezone(
            offset=datetime.timedelta(hours=5, minutes=30)
        )  # Indian timezone
        dt = datetime.datetime(2014, 11, 14, 10, 15, 11)
        utc_dt = to_utc_timezone(dt)
        self.assertEqual(utc_dt.tzinfo, datetime.timezone.utc)
        self.assertSequenceEqual(
            (
                utc_dt.year,
                utc_dt.month,
                utc_dt.day,
                utc_dt.hour,
                utc_dt.minute,
                utc_dt.second,
            ),
            (2014, 11, 14, 4, 45, 11),
        )

    def test_to_utc_timezone_with_utc_timezone(self):
        dt = datetime.datetime(2014, 11, 14, 10, 15, 11, tzinfo=datetime.timezone.utc)
        utc_dt = to_utc_timezone(dt)
        self.assertEqual(utc_dt.tzinfo, datetime.timezone.utc)
        self.assertSequenceEqual(
            (
                utc_dt.year,
                utc_dt.month,
                utc_dt.day,
                utc_dt.hour,
                utc_dt.minute,
                utc_dt.second,
            ),
            (2014, 11, 14, 10, 15, 11),
        )

    def test_to_utc_timezone_with_nepali_timezone(self):
        dt = datetime.datetime(2014, 11, 14, 10, 15, 11, tzinfo=NepaliTimeZone())
        utc_dt = to_utc_timezone(dt)
        self.assertEqual(utc_dt.tzinfo, datetime.timezone.utc)
        self.assertSequenceEqual(
            (
                utc_dt.year,
                utc_dt.month,
                utc_dt.day,
                utc_dt.hour,
                utc_dt.minute,
                utc_dt.second,
            ),
            (2014, 11, 14, 4, 30, 11),
        )

    # to_nepali_timezone
    def test_to_nepali_timezone_invalid_input_returns_as_it_is(self):
        self.assertEqual(to_nepali_timezone("test"), "test")  # type: ignore

    def test_to_nepali_timezone_without_timezone(self):
        dt = datetime.datetime(2014, 11, 14, 10, 15, 11)
        nepali_dt = to_nepali_timezone(dt)
        self.assertEqual(nepali_dt.tzinfo, NepaliTimeZone())
        self.assertSequenceEqual(
            (dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second),
            (2014, 11, 14, 10, 15, 11),
        )

    def test_to_nepali_timezone_with_utc_timezone(self):
        dt = datetime.datetime(2014, 11, 14, 4, 30, 11, tzinfo=datetime.timezone.utc)
        nepali_dt = to_nepali_timezone(dt)
        self.assertEqual(nepali_dt.tzinfo, NepaliTimeZone())
        self.assertSequenceEqual(
            (
                nepali_dt.year,
                nepali_dt.month,
                nepali_dt.day,
                nepali_dt.hour,
                nepali_dt.minute,
                nepali_dt.second,
            ),
            (2014, 11, 14, 10, 15, 11),
        )

    def test_to_nepali_timezone_with_nepali_timezone(self):
        dt = datetime.datetime(2014, 11, 14, 10, 15, 11, tzinfo=NepaliTimeZone())
        nepali_dt = to_nepali_timezone(dt)
        self.assertEqual(nepali_dt.tzinfo, NepaliTimeZone())
        self.assertSequenceEqual(
            (
                nepali_dt.year,
                nepali_dt.month,
                nepali_dt.day,
                nepali_dt.hour,
                nepali_dt.minute,
                nepali_dt.second,
            ),
            (2014, 11, 14, 10, 15, 11),
        )



"""
To run only this unit test use the command below.

    python -m unittest nepali/tests/test_number.py -v
"""
import sys
import unittest

from nepali import number
from nepali.number import NepaliNumber, nepalinumber


class TestNumber(unittest.TestCase):
    def test_number_english_to_nepali(self):
        self.assertEqual(
            number.english_to_nepali("0123456789०१२३४५६७८९hello"),
            "०१२३४५६७८९०१२३४५६७८९hello",
        )
        self.assertEqual(
            NepaliNumber.convert("0123456789०१२३४५६७८९"), "०१२३४५६७८९०१२३४५६७८९"
        )

    def test_number_nepali_to_english(self):
        self.assertEqual(
            number.nepali_to_english("0123456789०१२३४५६७८९hello"),
            "01234567890123456789hello",
        )
        self.assertEqual(
            NepaliNumber.revert("0123456789०१२३४५६७८९"), "01234567890123456789"
        )

    def test_number_add_comma(self):
        self.assertEqual(number.add_comma("123456789"), "12,34,56,789")
        self.assertEqual(NepaliNumber.add_comma("123456789"), "12,34,56,789")

    def test_number_add_comma_english(self):
        self.assertEqual(number.add_comma_english("123456789"), "123,456,789")
        self.assertEqual(NepaliNumber.add_comma_english("123456789"), "123,456,789")

    def test_number_add_comma_with_convert_True(self):
        self.assertEqual(number.add_comma("123456789", convert=True), "१२,३४,५६,७८९")

    def test_number_add_comma_with_convert_False(self):
        self.assertEqual(number.add_comma("123456789", convert=False), "12,34,56,789")

    def test_number_convert_and_add_comma(self):
        self.assertEqual(number.convert_and_add_comma("123456789"), "१२,३४,५६,७८९")
        self.assertEqual(
            NepaliNumber.convert_and_add_comma("123456789"), "१२,३४,५६,७८९"
        )


# to run this test suite exclusively
# python -m unittest nepali.tests.test_number.TestNepaliNumber -v
class TestNepaliNumberArithmeticOperations(unittest.TestCase):
    """
    Contains test cases related to nepalinumber
    arithmetic operations
    """

    @classmethod
    def setUpClass(cls):
        """
        Test fixtures
        """
        # positive integers
        cls.nepalinumber_integer_20 = number.nepalinumber(20)
        cls.nepalinumber_integer_15 = number.nepalinumber(15)
        cls.nepalinumber_integer_11 = number.nepalinumber(11)
        cls.nepalinumber_integer_10 = number.nepalinumber(10)
        cls.nepalinumber_integer_5 = number.nepalinumber(5)
        cls.nepalinumber_integer_4 = number.nepalinumber(4)
        cls.nepalinumber_integer_3 = number.nepalinumber(3)
        cls.nepalinumber_integer_2 = number.nepalinumber(2)
        cls.nepalinumber_integer_1 = number.nepalinumber(1)
        # positive floats
        cls.nepalinumber_float_21_21 = number.nepalinumber(21.21)
        cls.nepalinumber_float_20_2 = number.nepalinumber(20.2)
        cls.nepalinumber_float_20_1 = number.nepalinumber(20.1)
        cls.nepalinumber_float_15_6 = number.nepalinumber(15.6)
        cls.nepalinumber_float_15_5 = number.nepalinumber(15.5)
        cls.nepalinumber_float_10_9 = number.nepalinumber(10.9)
        cls.nepalinumber_float_10_0_cont = number.nepalinumber(
            20.1 - 10.1
        )  # 10.000...2
        cls.nepalinumber_float_10_1 = number.nepalinumber(10.1)
        cls.nepalinumber_float_9_9 = number.nepalinumber(9.9)
        cls.nepalinumber_float_5_5 = number.nepalinumber(5.5)
        cls.nepalinumber_float_5_25 = number.nepalinumber(5.25)
        cls.nepalinumber_float_4_6 = number.nepalinumber(4.6)
        cls.nepalinumber_float_4_5 = number.nepalinumber(4.5)
        cls.nepalinumber_float_3_5 = number.nepalinumber(3.5)
        cls.nepalinumber_float_2_5 = number.nepalinumber(2.5)
        cls.nepalinumber_float_2_25 = number.nepalinumber(2.25)
        cls.nepalinumber_float_1_5 = number.nepalinumber(1.5)
        cls.nepalinumber_float_1_25 = number.nepalinumber(1.25)
        cls.nepalinumber_float_0_5 = number.nepalinumber(0.5)
        cls.nepalinumber_float_0_1 = number.nepalinumber(0.1)
        cls.nepalinumber_float_0_0_9 = number.nepalinumber(
            10.1 - 10
        )  # to lazy to type 0.099...
        # negative numbers
        cls.nepalinumber_negative_integer_20 = number.nepalinumber(-20)
        cls.nepalinumber_negative_integer_15 = number.nepalinumber(-15)
        cls.nepalinumber_negative_integer_11 = number.nepalinumber(-11)
        cls.nepalinumber_negative_integer_10 = number.nepalinumber(-10)
        cls.nepalinumber_negative_integer_5 = number.nepalinumber(-5)
        cls.nepalinumber_negative_integer_4 = number.nepalinumber(-4)
        cls.nepalinumber_negative_integer_3 = number.nepalinumber(-3)
        cls.nepalinumber_negative_integer_2 = number.nepalinumber(-2)
        cls.nepalinumber_negative_integer_1 = number.nepalinumber(-1)
        # negative floats
        cls.nepalinumber_negative_float_21_21 = number.nepalinumber(-21.21)
        cls.nepalinumber_negative_float_20_2 = number.nepalinumber(-20.2)
        cls.nepalinumber_negative_float_20_1 = number.nepalinumber(-20.1)
        cls.nepalinumber_negative_float_20_0 = number.nepalinumber(-20.0)
        cls.nepalinumber_negative_float_15_6 = number.nepalinumber(-15.6)
        cls.nepalinumber_negative_float_15_5 = number.nepalinumber(-15.5)
        cls.nepalinumber_negative_float_10_1 = number.nepalinumber(-10.1)
        cls.nepalinumber_negative_float_9_9 = number.nepalinumber(-9.9)
        cls.nepalinumber_negative_float_5_5 = number.nepalinumber(-5.5)
        cls.nepalinumber_negative_float_5_4 = number.nepalinumber(-5.4)
        cls.nepalinumber_negative_float_4_5 = number.nepalinumber(-4.5)
        cls.nepalinumber_negative_float_2_5 = number.nepalinumber(-2.5)
        cls.nepalinumber_negative_float_2_25 = number.nepalinumber(-2.25)
        cls.nepalinumber_negative_float_1_5 = number.nepalinumber(-1.5)
        cls.nepalinumber_negative_float_1_25 = number.nepalinumber(-1.25)
        cls.nepalinumber_negative_float_0_5 = number.nepalinumber(-0.5)
        cls.nepalinumber_negative_float_0_0_9 = number.nepalinumber(
            10 - 10.1
        )  # to lazy to type -0.099...
        # zero
        cls.nepalinumber_zero = number.nepalinumber(0)
        # objects
        cls.random_object = object
        # string
        cls.random_string = "test"

    # addition
    # nepalinumber positive integer addition tests
    def test_nepalinumber_integer_is_addable_to_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_integer_10 + 10, self.nepalinumber_integer_20
        )

    def test_nepalinumber_integer_is_addable_to_positive_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 + 10.1, self.nepalinumber_float_20_1
        )

    def test_nepalinumber_integer_is_addable_to_negative_integer(self):
        self.assertEqual(self.nepalinumber_integer_10 + (-10), self.nepalinumber_zero)

    def test_nepalinumber_integer_is_addable_to_negative_integer_to_give_negative_nepali_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 + (-20), self.nepalinumber_negative_integer_10
        )

    def test_nepalinumber_integer_is_addable_to_negative_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 + (-5.5), self.nepalinumber_float_4_5
        )

    def test_nepalinumber_integer_is_addable_to_negative_float_to_give_negative_nepali_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 + (-15.5), self.nepalinumber_negative_float_5_5
        )

    def test_nepalinumber_integer_is_addable_to_positive_nepali_number_integer(self):
        self.assertEqual(
            self.nepalinumber_integer_10 + self.nepalinumber_integer_10,
            self.nepalinumber_integer_20,
        )

    def test_nepalinumber_integer_is_addable_to_positive_nepali_number_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 + self.nepalinumber_float_10_1,
            self.nepalinumber_float_20_1,
        )

    def test_nepalinumber_integer_is_addable_to_negative_nepali_number_integer(self):
        self.assertEqual(
            self.nepalinumber_integer_10 + self.nepalinumber_negative_integer_10,
            self.nepalinumber_zero,
        )

    def test_nepalinumber_integer_is_addable_to_negative_nepali_number_integer_to_give_negative_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 + self.nepalinumber_negative_integer_20,
            self.nepalinumber_negative_integer_10,
        )

    def test_nepalinumber_integer_is_addable_to_negative_nepali_number_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 + self.nepalinumber_negative_float_5_5,
            self.nepalinumber_float_4_5,
        )

    def test_nepalinumber_integer_is_addable_to_negative_nepali_number_float_to_give_negative_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 + self.nepalinumber_negative_float_15_5,
            self.nepalinumber_negative_float_5_5,
        )

    # adding the nepali number to other numbers tests
    def test_positive_integer_is_addable_to_nepalinumber_integer(self):
        self.assertEqual(
            10 + self.nepalinumber_integer_10, self.nepalinumber_integer_20
        )

    def test_positive_float_is_addable_to_nepalinumber_integer(self):
        self.assertEqual(
            10.1 + self.nepalinumber_integer_10, self.nepalinumber_float_20_1
        )

    def test_negative_integer_is_addable_to_nepalinumber_integer(self):
        self.assertEqual(-10 + self.nepalinumber_integer_10, self.nepalinumber_zero)

    def test_negative_integer_is_addable_to_nepalinumber_integer_to_give_negative_nepali_integer(
        self,
    ):
        self.assertEqual(
            (-20) + self.nepalinumber_integer_10, self.nepalinumber_negative_integer_10
        )

    def test_negative_float_is_addable_to_nepalinumber_integer(self):
        self.assertEqual(
            (-5.5) + self.nepalinumber_integer_10, self.nepalinumber_float_4_5
        )

    def test_negative_float_is_addable_to_nepalinumber_integer_to_give_negative_nepali_float(
        self,
    ):
        self.assertEqual(
            (-15.5) + self.nepalinumber_integer_10, self.nepalinumber_negative_float_5_5
        )

    # nepalinumber negative integer addition tests
    def test_negative_nepalinumber_integer_is_addable_to_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 + 10, self.nepalinumber_zero
        )

    def test_negative_nepalinumber_integer_is_addable_to_positive_float(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 + 10.1, self.nepalinumber_float_0_0_9
        )

    def test_negative_nepalinumber_integer_is_addable_to_positive_integer_to_give_positive_nepali_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 + 20, self.nepalinumber_integer_10
        )

    def test_negative_nepalinumber_integer_is_addable_to_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 + (-10),
            self.nepalinumber_negative_float_20_0,
        )

    def test_negative_nepalinumber_integer_is_addable_to_negative_float(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 + (-5.5),
            self.nepalinumber_negative_float_15_5,
        )

    def test_negative_nepalinumber_integer_is_addable_to_positive_float_to_give_positive_nepali_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 + 15.5, self.nepalinumber_float_5_5
        )

    def test_negative_nepalinumber_integer_is_addable_to_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 + self.nepalinumber_integer_10,
            self.nepalinumber_zero,
        )

    def test_negative_nepalinumber_integer_is_addable_to_positive_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 + self.nepalinumber_float_10_1,
            self.nepalinumber_float_0_0_9,
        )

    def test_negative_nepalinumber_integer_is_addable_to_negative_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10
            + self.nepalinumber_negative_integer_10,
            self.nepalinumber_negative_integer_20,
        )

    def test_negative_nepalinumber_integer_is_addable_to_positive_nepali_number_integer_to_give_positive_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 + self.nepalinumber_integer_20,
            self.nepalinumber_integer_10,
        )

    def test_negative_nepalinumber_integer_is_addable_to_negative_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10
            + self.nepalinumber_negative_float_5_5,
            self.nepalinumber_negative_float_15_5,
        )

    def test_nepalinumber_integer_is_addable_to_positive_nepali_number_float_to_give_positive_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 + self.nepalinumber_float_20_1,
            self.nepalinumber_float_10_0_cont,
        )

    # nepalinumber positive float addition tests
    def test_nepalinumber_float_is_addable_to_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 + 10, self.nepalinumber_float_20_1
        )

    def test_nepalinumber_float_is_addable_to_positive_float(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 + 10.1, self.nepalinumber_float_20_2
        )

    def test_nepalinumber_float_is_addable_to_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 + (-10), self.nepalinumber_float_0_0_9
        )

    def test_nepalinumber_float_is_addable_to_negative_integer_to_give_negative_nepali_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_10_9 + (-21), self.nepalinumber_negative_float_10_1
        )

    def test_nepalinumber_float_is_addable_to_negative_float(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 + (-5.6), self.nepalinumber_float_4_5
        )

    def test_nepalinumber_float_is_addable_to_negative_float_to_give_negative_nepali_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_10_1 + (-15.6), self.nepalinumber_negative_float_5_5
        )

    def test_nepalinumber_float_is_addable_to_positive_nepali_number_integer(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 + self.nepalinumber_integer_10,
            self.nepalinumber_float_20_1,
        )

    def test_nepalinumber_float_is_addable_to_positive_nepali_number_float(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 + self.nepalinumber_float_10_1,
            self.nepalinumber_float_20_2,
        )

    def test_nepalinumber_float_is_addable_to_negative_nepali_number_integer(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 + self.nepalinumber_negative_integer_10,
            self.nepalinumber_float_0_0_9,
        )

    def test_nepalinumber_float_is_addable_to_negative_nepali_number_integer_to_give_negative_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_10_1 + self.nepalinumber_negative_integer_20,
            self.nepalinumber_negative_float_9_9,
        )

    def test_nepalinumber_float_is_addable_to_negative_nepali_number_float(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 + self.nepalinumber_negative_float_5_5,
            self.nepalinumber_float_4_6,
        )

    def test_nepalinumber_float_is_addable_to_negative_nepali_number_float_to_give_negative_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_10_1 + self.nepalinumber_negative_float_15_5,
            self.nepalinumber_negative_float_5_4,
        )

    # nepalinumber negative float addition tests
    def test_negative_nepalinumber_float_is_addable_to_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 + 10,
            self.nepalinumber_negative_float_0_0_9,
        )

    def test_negative_nepalinumber_float_is_addable_to_positive_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 + 10.1, self.nepalinumber_zero
        )

    def test_negative_nepalinumber_float_is_addable_to_positive_integer_to_give_positive_nepali_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 + 20, self.nepalinumber_float_9_9
        )

    def test_negative_nepalinumber_float_is_addable_to_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 + (-10),
            self.nepalinumber_negative_float_20_1,
        )

    def test_negative_nepalinumber_float_is_addable_to_negative_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 + (-5.4),
            self.nepalinumber_negative_float_15_5,
        )

    def test_negative_nepalinumber_float_is_addable_to_positive_float_to_give_positive_nepali_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 + 15.6, self.nepalinumber_float_5_5
        )

    def test_negative_nepalinumber_float_is_addable_to_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 + self.nepalinumber_integer_10,
            self.nepalinumber_negative_float_0_0_9,
        )

    def test_negative_nepalinumber_float_is_addable_to_positive_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 + self.nepalinumber_float_10_1,
            self.nepalinumber_zero,
        )

    def test_negative_nepalinumber_float_is_addable_to_negative_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1
            + self.nepalinumber_negative_integer_10,
            self.nepalinumber_negative_float_20_1,
        )

    def test_negative_nepalinumber_float_is_addable_to_positive_nepali_number_integer_to_give_positive_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 + self.nepalinumber_integer_20,
            self.nepalinumber_float_9_9,
        )

    def test_negative_nepalinumber_float_is_addable_to_negative_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1
            + self.nepalinumber_negative_float_5_5,
            self.nepalinumber_negative_float_15_6,
        )

    def test_nepalinumber_float_is_addable_to_positive_nepali_number_float_to_give_positive_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 + self.nepalinumber_float_20_1,
            self.nepalinumber_float_10_0_cont,
        )

    # addition negative testing
    def test_nepalinumber_throws_error_when_objects_are_added_other_than_int_or_float(
        self,
    ):
        self.assertNotIsInstance(self.random_object, int)
        self.assertNotIsInstance(self.random_object, float)

        with self.assertRaises(TypeError) as te:
            _ = self.nepalinumber_integer_10 + self.random_object

        self.assertEqual(
            str(te.exception),
            "unsupported operand type(s) for +: 'nepalinumber' and 'type'",
        )

    def test_nepalinumber_throws_error_when_added_with_objects_other_than_int_or_float(
        self,
    ):
        self.assertNotIsInstance(self.random_object, int)
        self.assertNotIsInstance(self.random_object, float)

        with self.assertRaises(TypeError) as te:
            _ = self.random_object + self.nepalinumber_integer_10

        self.assertEqual(
            str(te.exception),
            "unsupported operand type(s) for +: 'type' and 'nepalinumber'",
        )

    # subtraction
    # nepalinumber positive integer subtraction tests
    def test_nepalinumber_integer_is_subtractable_by_positive_integer(self):
        self.assertEqual(self.nepalinumber_integer_10 - 10, self.nepalinumber_zero)

    def test_nepalinumber_integer_is_subtractable_by_positive_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 - 5.5, self.nepalinumber_float_4_5
        )

    def test_nepalinumber_integer_is_subtractable_by_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_integer_10 - (-10), self.nepalinumber_integer_20
        )

    def test_nepalinumber_integer_is_subtractable_by_negative_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 - (-5.5), self.nepalinumber_float_15_5
        )

    def test_nepalinumber_integer_is_subtractable_by_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 - self.nepalinumber_integer_10,
            self.nepalinumber_zero,
        )

    def test_nepalinumber_integer_is_subtractable_by_positive_nepali_number_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 - self.nepalinumber_float_5_5,
            self.nepalinumber_float_4_5,
        )

    def test_nepalinumber_integer_is_subtractable_by_negative_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 - self.nepalinumber_negative_integer_10,
            self.nepalinumber_integer_20,
        )

    def test_nepalinumber_integer_is_subtractable_by_negative_nepali_number_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 - self.nepalinumber_negative_float_5_5,
            self.nepalinumber_float_15_5,
        )

    # subtracting the nepali number to other numbers tests
    def test_positive_integer_is_subtractable_by_nepalinumber_integer(self):
        self.assertEqual(10 - self.nepalinumber_integer_10, self.nepalinumber_zero)

    def test_positive_float_is_subtractable_by_nepalinumber_integer(self):
        self.assertEqual(
            10.1 - self.nepalinumber_integer_10, self.nepalinumber_float_0_0_9
        )

    def test_negative_integer_is_subtractable_by_nepalinumber_integer(self):
        self.assertEqual(
            (-10) - self.nepalinumber_integer_10, self.nepalinumber_negative_integer_20
        )

    def test_negative_float_is_subtractable_by_nepalinumber_integer(self):
        self.assertEqual(
            (-5.5) - self.nepalinumber_integer_10, self.nepalinumber_negative_float_15_5
        )

    def test_negative_float_is_subtractable_by_nepalinumber_integer_to_give_negative_nepali_float(
        self,
    ):
        self.assertEqual(
            (-5.5) - self.nepalinumber_integer_10, self.nepalinumber_negative_float_15_5
        )

    # nepalinumber negative integer subtraction tests
    def test_negative_nepalinumber_integer_is_subtractable_by_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 - 10,
            self.nepalinumber_negative_integer_20,
        )

    def test_negative_nepalinumber_integer_is_subtractable_by_positive_float(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 - 10.1,
            self.nepalinumber_negative_float_20_1,
        )

    def test_negative_nepalinumber_integer_is_subtractable_by_negative_integer_to_give_positive_nepali_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 - (-20), self.nepalinumber_integer_10
        )

    def test_negative_nepalinumber_integer_is_subtractable_by_negative_float(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 - (-5.5),
            self.nepalinumber_negative_float_4_5,
        )

    def test_negative_nepalinumber_integer_is_subtractable_by_negative_float_to_give_positive_nepali_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 - (-15.5), self.nepalinumber_float_5_5
        )

    def test_negative_nepalinumber_integer_is_subtractable_by_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 - self.nepalinumber_integer_10,
            self.nepalinumber_negative_float_20_0,
        )

    def test_negative_nepalinumber_integer_is_subtractable_by_positive_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 - self.nepalinumber_float_10_1,
            self.nepalinumber_negative_float_20_1,
        )

    def test_negative_nepalinumber_integer_is_subtractable_by_negative_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10
            - self.nepalinumber_negative_integer_10,
            self.nepalinumber_zero,
        )

    def test_negative_nepalinumber_integer_is_subtractable_by_negative_nepali_number_integer_to_give_positive_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10
            - self.nepalinumber_negative_integer_20,
            self.nepalinumber_integer_10,
        )

    def test_negative_nepalinumber_integer_is_subtractable_by_negative_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10
            - self.nepalinumber_negative_float_5_5,
            self.nepalinumber_negative_float_4_5,
        )

    def test_nepalinumber_integer_is_subtractable_by_negative_nepali_number_float_to_give_positive_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1
            - self.nepalinumber_negative_float_20_1,
            self.nepalinumber_float_10_0_cont,
        )

    # nepalinumber positive float subtraction tests
    def test_nepalinumber_float_is_subtractable_by_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 - 10, self.nepalinumber_float_0_0_9
        )

    def test_nepalinumber_float_is_subtractable_by_positive_float(self):
        self.assertEqual(self.nepalinumber_float_10_1 - 10.1, self.nepalinumber_zero)

    def test_nepalinumber_float_is_subtractable_by_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 - (-10), self.nepalinumber_float_20_1
        )

    def test_nepalinumber_float_is_subtractable_by_negative_float(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 - (-5.4), self.nepalinumber_float_15_5
        )

    def test_nepalinumber_float_is_subtractable_by_positive_nepali_number_integer(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 - self.nepalinumber_integer_10,
            self.nepalinumber_float_0_0_9,
        )

    def test_nepalinumber_float_is_subtractable_by_positive_nepali_number_float(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 - self.nepalinumber_float_10_1,
            self.nepalinumber_zero,
        )

    def test_nepalinumber_float_is_subtractable_by_negative_nepali_number_integer(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 - self.nepalinumber_negative_integer_10,
            self.nepalinumber_float_20_1,
        )

    def test_nepalinumber_float_is_subtractable_by_positive_nepali_number_integer_to_give_negative_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_10_1 - self.nepalinumber_integer_20,
            self.nepalinumber_negative_float_9_9,
        )

    def test_nepalinumber_float_is_subtractable_by_negative_nepali_number_float(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 - self.nepalinumber_negative_float_5_5,
            self.nepalinumber_float_15_6,
        )

    # nepalinumber negative float subtraction tests
    def test_negative_nepalinumber_float_is_subtractable_by_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 - 10,
            self.nepalinumber_negative_float_20_1,
        )

    def test_negative_nepalinumber_float_is_subtractable_by_positive_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 - 5.4,
            self.nepalinumber_negative_float_15_5,
        )

    def test_negative_nepalinumber_float_is_subtractable_by_negative_integer_to_give_positive_nepali_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 - (-20), self.nepalinumber_float_9_9
        )

    def test_negative_nepalinumber_float_is_subtractable_by_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 - (-10),
            self.nepalinumber_negative_float_0_0_9,
        )

    def test_negative_nepalinumber_float_is_subtractable_by_negative_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 - (-5.6),
            self.nepalinumber_negative_float_4_5,
        )

    def test_negative_nepalinumber_float_is_subtractable_by_negative_float_to_give_positive_nepali_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 - (-15.6), self.nepalinumber_float_5_5
        )

    def test_negative_nepalinumber_float_is_subtractable_by_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 - self.nepalinumber_integer_10,
            self.nepalinumber_negative_float_20_1,
        )

    def test_negative_nepalinumber_float_is_subtractable_by_positive_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 - self.nepalinumber_float_5_5,
            self.nepalinumber_negative_float_15_6,
        )

    def test_negative_nepalinumber_float_is_subtractable_by_negative_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1
            - self.nepalinumber_negative_integer_10,
            self.nepalinumber_negative_float_0_0_9,
        )

    def test_negative_nepalinumber_float_is_subtractable_by_negative_nepali_number_integer_to_give_positive_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1
            - self.nepalinumber_negative_integer_20,
            self.nepalinumber_float_9_9,
        )

    def test_negative_nepalinumber_float_is_subtractable_by_negative_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1
            - self.nepalinumber_negative_float_10_1,
            self.nepalinumber_zero,
        )

    def test_nepalinumber_float_is_subtractable_by_negative_nepali_number_float_to_give_positive_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1
            - self.nepalinumber_negative_float_20_1,
            self.nepalinumber_float_10_0_cont,
        )

    # subtraction negative testing
    def test_nepalinumber_throws_error_when_subtracted_by_object_other_than_int_or_float(
        self,
    ):
        self.assertNotIsInstance(self.random_object, int)
        self.assertNotIsInstance(self.random_object, float)

        with self.assertRaises(TypeError) as te:
            _ = self.nepalinumber_integer_10 - self.random_object

        self.assertEqual(
            str(te.exception),
            "unsupported operand type(s) for -: 'nepalinumber' and 'type'",
        )

    def test_nepalinumber_throws_error_when_subtracted_to_objects_other_than_int_or_float(
        self,
    ):
        self.assertNotIsInstance(self.random_object, int)
        self.assertNotIsInstance(self.random_object, float)

        with self.assertRaises(TypeError) as te:
            _ = self.random_object - self.nepalinumber_integer_10

        self.assertEqual(
            str(te.exception),
            "unsupported operand type(s) for -: 'type' and 'nepalinumber'",
        )

    # multiplication
    # nepalinumber positive integer multiplication tests
    def test_nepalinumber_integer_is_multiplicable_to_positive_integer(self):
        self.assertEqual(self.nepalinumber_integer_10 * 2, self.nepalinumber_integer_20)

    def test_nepalinumber_integer_is_multiplicable_to_positive_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 * 1.5, self.nepalinumber_integer_15
        )

    def test_nepalinumber_integer_is_multiplicable_to_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_integer_10 * (-2), self.nepalinumber_negative_integer_20
        )

    def test_nepalinumber_integer_is_multiplicable_to_negative_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 * (-1.5), self.nepalinumber_negative_integer_15
        )

    def test_nepalinumber_integer_is_multiplicable_to_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 * self.nepalinumber_integer_2,
            self.nepalinumber_integer_20,
        )

    def test_nepalinumber_integer_is_multiplicable_to_positive_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 * self.nepalinumber_float_1_5,
            self.nepalinumber_integer_15,
        )

    def test_nepalinumber_integer_is_multiplicable_to_negative_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 * self.nepalinumber_negative_integer_2,
            self.nepalinumber_negative_integer_20,
        )

    def test_nepalinumber_integer_is_multiplicable_to_negative_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 * self.nepalinumber_negative_float_1_5,
            self.nepalinumber_negative_integer_15,
        )

    # multiplying the nepali number to other numbers tests
    def test_positive_integer_is_multiplicable_to_nepalinumber_integer(self):
        self.assertEqual(2 * self.nepalinumber_integer_10, self.nepalinumber_integer_20)

    def test_positive_float_is_multiplicable_to_nepalinumber_integer(self):
        self.assertEqual(
            1.5 * self.nepalinumber_integer_10, self.nepalinumber_integer_15
        )

    def test_negative_integer_is_multiplicable_to_nepalinumber_integer(self):
        self.assertEqual(
            -2 * self.nepalinumber_integer_10, self.nepalinumber_negative_integer_20
        )

    def test_negative_integer_is_multiplicable_to_nepalinumber_integer_to_give_negative_nepali_integer(
        self,
    ):
        self.assertEqual(
            -2 * self.nepalinumber_integer_10, self.nepalinumber_negative_integer_20
        )

    def test_negative_float_is_multiplicable_to_nepalinumber_integer(self):
        self.assertEqual(
            (-1.5) * self.nepalinumber_integer_10, self.nepalinumber_negative_integer_15
        )

    # nepalinumber negative integer multiplication tests
    def test_negative_nepalinumber_integer_is_multiplicable_to_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 * 2,
            self.nepalinumber_negative_integer_20,
        )

    def test_negative_nepalinumber_integer_is_multiplicable_to_positive_float(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 * 1.5,
            self.nepalinumber_negative_integer_15,
        )

    def test_negative_nepalinumber_integer_is_multiplicable_to_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 * (-2),
            self.nepalinumber_integer_20,
        )

    def test_negative_nepalinumber_integer_is_multiplicable_to_negative_float(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 * (-1.5),
            self.nepalinumber_integer_15,
        )

    def test_negative_nepalinumber_integer_is_multiplicable_to_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 * self.nepalinumber_integer_2,
            self.nepalinumber_negative_integer_20,
        )

    def test_negative_nepalinumber_integer_is_multiplicable_to_positive_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 * self.nepalinumber_float_1_5,
            self.nepalinumber_negative_integer_15,
        )

    def test_negative_nepalinumber_integer_is_multiplicable_to_negative_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10
            * self.nepalinumber_negative_integer_2,
            self.nepalinumber_integer_20,
        )

    def test_negative_nepalinumber_integer_is_multiplicable_to_negative_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10
            * self.nepalinumber_negative_float_1_5,
            self.nepalinumber_integer_15,
        )

    # nepalinumber positive float multiplication tests
    def test_nepalinumber_float_is_multiplicable_to_positive_integer(self):
        self.assertEqual(self.nepalinumber_float_1_5 * 10, self.nepalinumber_integer_15)

    def test_nepalinumber_float_is_multiplicable_to_positive_float(self):
        self.assertEqual(
            self.nepalinumber_float_10_1 * 2.1, self.nepalinumber_float_21_21
        )

    def test_nepalinumber_float_is_multiplicable_to_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_float_1_5 * (-3), self.nepalinumber_negative_float_4_5
        )

    def test_nepalinumber_float_is_multiplicable_to_negative_float(self):
        self.assertEqual(
            self.nepalinumber_float_4_5 * (-2.2), self.nepalinumber_negative_float_9_9
        )

    def test_nepalinumber_float_is_multiplicable_to_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_10_1 * self.nepalinumber_integer_2,
            self.nepalinumber_float_20_2,
        )

    def test_nepalinumber_float_is_multiplicable_to_positive_nepali_number_float(self):
        self.assertEqual(
            self.nepalinumber_float_1_5 * self.nepalinumber_float_3_5,
            self.nepalinumber_float_5_25,
        )

    def test_nepalinumber_float_is_multiplicable_to_negative_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_10_1 * self.nepalinumber_negative_integer_2,
            self.nepalinumber_negative_float_20_2,
        )

    def test_nepalinumber_float_is_multiplicable_to_negative_nepali_number_float(self):
        self.assertEqual(
            self.nepalinumber_float_1_5 * self.nepalinumber_negative_float_1_5,
            self.nepalinumber_negative_float_2_25,
        )

    # nepalinumber negative float addition tests
    def test_negative_nepalinumber_float_is_multiplicable_to_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 * 2,
            self.nepalinumber_negative_float_20_2,
        )

    def test_negative_nepalinumber_float_is_multiplicable_to_positive_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_1_5 * 1.5,
            self.nepalinumber_negative_float_2_25,
        )

    def test_negative_nepalinumber_float_is_multiplicable_to_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 * (-2),
            self.nepalinumber_float_20_2,
        )

    def test_negative_nepalinumber_float_is_multiplicable_to_negative_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_1_5 * (-3.5),
            self.nepalinumber_float_5_25,
        )

    def test_negative_nepalinumber_float_is_multiplicable_to_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1 * self.nepalinumber_integer_2,
            self.nepalinumber_negative_float_20_2,
        )

    def test_negative_nepalinumber_float_is_multiplicable_to_positive_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_1_5 * self.nepalinumber_float_1_5,
            self.nepalinumber_negative_float_2_25,
        )

    def test_negative_nepalinumber_float_is_multiplicable_to_negative_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_10_1
            * self.nepalinumber_negative_integer_2,
            self.nepalinumber_float_20_2,
        )

    def test_negative_nepalinumber_float_is_multiplicable_to_negative_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_1_5 * self.nepalinumber_negative_float_1_5,
            self.nepalinumber_float_2_25,
        )

    # string tests
    def test_nepalinumber_integer_can_be_multiplied_with_string(self):
        self.assertEqual(
            self.nepalinumber_integer_2 * self.random_string, 2 * self.random_string
        )

    def test_string_can_be_multiplied_by_nepalinumber_integer(self):
        self.assertEqual(
            self.random_string * self.nepalinumber_integer_2, self.random_string * 2
        )

    def test_nepalinumber_negative_integer_on_multiplication_with_string_gives_empty_string(
        self,
    ):
        self.assertEqual(self.nepalinumber_negative_integer_2 * self.random_string, "")

    def test_string_can_be_multiplied_by_negative_nepalinumber_integer_to_give_empty_string(
        self,
    ):
        self.assertEqual(self.random_string * self.nepalinumber_negative_integer_2, "")

    def test_nepalinumber_float_throws_error_when_multiplied_with_string(self):
        with self.assertRaises(TypeError) as te:
            _ = self.nepalinumber_float_0_1 * self.random_string

        self.assertEqual(
            str(te.exception),
            "can't multiply sequence by non-int of type 'nepalinumber'",
        )

    def test_nepalinumber_float_throws_error_when_multiplied_to_string(self):
        with self.assertRaises(TypeError) as te:
            _ = self.random_string * self.nepalinumber_float_0_0_9

        self.assertEqual(
            str(te.exception),
            "can't multiply sequence by non-int of type 'nepalinumber'",
        )

    # multiplication negative testing
    def test_nepalinumber_throws_error_when_objects_are_multiplied_other_than_int_or_float(
        self,
    ):
        self.assertNotIsInstance(self.random_object, int)
        self.assertNotIsInstance(self.random_object, float)

        with self.assertRaises(TypeError) as te:
            _ = self.nepalinumber_integer_10 * self.random_object

        self.assertEqual(
            str(te.exception),
            "unsupported operand type(s) for *: 'nepalinumber' and 'type'",
        )

    def test_nepalinumber_throws_error_when_multiplied_with_objects_other_than_int_or_float(
        self,
    ):
        self.assertNotIsInstance(self.random_object, int)
        self.assertNotIsInstance(self.random_object, float)

        with self.assertRaises(TypeError) as te:
            _ = self.random_object + self.nepalinumber_integer_10

        self.assertEqual(
            str(te.exception),
            "unsupported operand type(s) for +: 'type' and 'nepalinumber'",
        )

    # division
    # nepalinumber positive integer division tests
    def test_nepalinumber_integer_is_divisible_by_positive_integer(self):
        self.assertEqual(self.nepalinumber_integer_20 / 2, self.nepalinumber_integer_10)

    def test_nepalinumber_integer_is_divisible_by_positive_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 / 2.5, self.nepalinumber_integer_4
        )

    def test_nepalinumber_integer_is_divisible_by_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_integer_20 / (-2), self.nepalinumber_negative_integer_10
        )

    def test_nepalinumber_integer_is_divisible_by_negative_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 / (-2.5), self.nepalinumber_negative_integer_4
        )

    def test_nepalinumber_integer_is_divisible_by_positive_nepali_number_integer(self):
        self.assertEqual(
            self.nepalinumber_integer_20 / self.nepalinumber_integer_10,
            self.nepalinumber_integer_2,
        )

    def test_nepalinumber_integer_is_divisible_by_positive_nepali_number_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 / self.nepalinumber_float_2_5,
            self.nepalinumber_integer_4,
        )

    def test_nepalinumber_integer_is_divisible_by_negative_nepali_number_integer(self):
        self.assertEqual(
            self.nepalinumber_integer_20 / self.nepalinumber_negative_integer_10,
            self.nepalinumber_negative_integer_2,
        )

    def test_nepalinumber_integer_is_divisible_by_negative_nepali_number_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 / self.nepalinumber_negative_float_2_5,
            self.nepalinumber_negative_integer_4,
        )

    # dividing other numbers by nepali number tests
    def test_positive_integer_is_divisible_by_nepalinumber_integer(self):
        self.assertEqual(20 / self.nepalinumber_integer_2, self.nepalinumber_integer_10)

    def test_positive_float_is_divisible_by_nepalinumber_integer(self):
        self.assertEqual(
            20.2 / self.nepalinumber_integer_2, self.nepalinumber_float_10_1
        )

    def test_negative_integer_is_divisible_by_nepalinumber_integer(self):
        self.assertEqual(
            -20 / self.nepalinumber_integer_10, self.nepalinumber_negative_integer_2
        )

    def test_negative_integer_is_divisible_by_negative_nepalinumber_integer_to_give_positive_nepali_integer(
        self,
    ):
        self.assertEqual(
            (-20) / self.nepalinumber_negative_integer_10, self.nepalinumber_integer_2
        )

    def test_negative_float_is_divisible_by_nepalinumber_integer(self):
        self.assertEqual(
            (-20.2) / self.nepalinumber_integer_2, self.nepalinumber_negative_float_10_1
        )

    def test_negative_float_is_divisible_by_negative_nepalinumber_integer_to_give_positive_nepali_float(
        self,
    ):
        self.assertEqual(
            (-20.2) / self.nepalinumber_negative_integer_2, self.nepalinumber_float_10_1
        )

    # nepalinumber negative integer division tests
    def test_negative_nepalinumber_integer_is_divisible_by_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 / 5,
            self.nepalinumber_negative_integer_2,
        )

    def test_negative_nepalinumber_integer_is_divisible_by_positive_float(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 / 2.5,
            self.nepalinumber_negative_integer_4,
        )

    def test_negative_nepalinumber_integer_is_divisible_by_negative_integer_to_give_positive_nepali_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 / -5, self.nepalinumber_integer_2
        )

    def test_negative_nepalinumber_integer_is_divisible_by_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 / (-5),
            self.nepalinumber_integer_2,
        )

    def test_negative_nepalinumber_integer_is_divisible_by_negative_float(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 / (-2.5),
            self.nepalinumber_integer_4,
        )

    def test_negative_nepalinumber_integer_is_divisible_by_negative_float_to_give_positive_nepali_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 / -2.5, self.nepalinumber_integer_4
        )

    def test_negative_nepalinumber_integer_is_divisible_by_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_20 / self.nepalinumber_integer_10,
            self.nepalinumber_negative_integer_2,
        )

    def test_negative_nepalinumber_integer_is_divisible_by_positive_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 / self.nepalinumber_float_2_5,
            self.nepalinumber_negative_integer_4,
        )

    def test_negative_nepalinumber_integer_is_divisible_by_negative_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_20
            / self.nepalinumber_negative_integer_10,
            self.nepalinumber_integer_2,
        )

    def test_negative_nepalinumber_integer_is_divisible_by_negative_nepali_number_integer_to_give_positive_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_20
            / self.nepalinumber_negative_integer_2,
            self.nepalinumber_integer_10,
        )

    def test_negative_nepalinumber_integer_is_divisible_by_negative_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10
            / self.nepalinumber_negative_float_2_5,
            self.nepalinumber_integer_4,
        )

    # nepalinumber positive float division tests
    def test_nepalinumber_float_is_divisible_by_positive_integer(self):
        self.assertEqual(self.nepalinumber_float_20_2 / 2, self.nepalinumber_float_10_1)

    def test_nepalinumber_float_is_divisible_by_positive_float(self):
        self.assertEqual(
            self.nepalinumber_float_20_2 / 10.1, self.nepalinumber_integer_2
        )

    def test_nepalinumber_float_is_divisible_by_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_float_20_2 / (-2), self.nepalinumber_negative_float_10_1
        )

    def test_nepalinumber_float_is_divisible_by_negative_float(self):
        self.assertEqual(
            self.nepalinumber_float_20_2 / (-10.1), self.nepalinumber_negative_integer_2
        )

    def test_nepalinumber_float_is_divisible_by_positive_nepali_number_integer(self):
        self.assertEqual(
            self.nepalinumber_float_20_2 / self.nepalinumber_integer_2,
            self.nepalinumber_float_10_1,
        )

    def test_nepalinumber_float_is_divisible_by_positive_nepali_number_float(self):
        self.assertEqual(
            self.nepalinumber_float_20_2 / self.nepalinumber_float_10_1,
            self.nepalinumber_integer_2,
        )

    def test_nepalinumber_float_is_divisible_by_negative_nepali_number_integer(self):
        self.assertEqual(
            self.nepalinumber_float_20_2 / self.nepalinumber_negative_integer_2,
            self.nepalinumber_negative_float_10_1,
        )

    def test_nepalinumber_float_is_divisible_by_negative_nepali_number_float(self):
        self.assertEqual(
            self.nepalinumber_float_20_2 / self.nepalinumber_negative_float_10_1,
            self.nepalinumber_negative_integer_2,
        )

    # nepalinumber negative float division tests
    def test_negative_nepalinumber_float_is_divisible_by_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2 / 2,
            self.nepalinumber_negative_float_10_1,
        )

    def test_negative_nepalinumber_float_is_divisible_by_positive_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2 / 10.1,
            self.nepalinumber_negative_integer_2,
        )

    def test_negative_nepalinumber_float_is_divisible_by_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2 / (-2),
            self.nepalinumber_float_10_1,
        )

    def test_negative_nepalinumber_float_is_divisible_by_negative_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2 / (-10.1),
            self.nepalinumber_integer_2,
        )

    def test_negative_nepalinumber_float_is_divisible_by_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2 / self.nepalinumber_integer_2,
            self.nepalinumber_negative_float_10_1,
        )

    def test_negative_nepalinumber_float_is_divisible_by_positive_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2 / self.nepalinumber_float_10_1,
            self.nepalinumber_negative_integer_2,
        )

    def test_negative_nepalinumber_float_is_divisible_by_negative_nepali_number_integer_to_give_positive_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2
            / self.nepalinumber_negative_integer_2,
            self.nepalinumber_float_10_1,
        )

    def test_negative_nepalinumber_float_is_divisible_by_negative_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2
            / self.nepalinumber_negative_float_10_1,
            self.nepalinumber_integer_2,
        )

    # division negative testing
    def test_nepalinumber_throws_error_when_divided_by_objects_other_than_int_or_float(
        self,
    ):
        self.assertNotIsInstance(self.random_object, int)
        self.assertNotIsInstance(self.random_object, float)

        with self.assertRaises(TypeError) as te:
            _ = self.nepalinumber_integer_10 / self.random_object

        self.assertEqual(
            str(te.exception),
            "unsupported operand type(s) for /: 'nepalinumber' and 'type'",
        )

    def test_nepalinumber_throws_error_when_divided_to_objects_other_than_int_or_float(
        self,
    ):
        self.assertNotIsInstance(self.random_object, int)
        self.assertNotIsInstance(self.random_object, float)

        with self.assertRaises(TypeError) as te:
            _ = self.random_object / self.nepalinumber_integer_10

        self.assertEqual(
            str(te.exception),
            "unsupported operand type(s) for /: 'type' and 'nepalinumber'",
        )

    def test_nepalinumber_throws_error_when_divided_by_zero(self):
        with self.assertRaises(ZeroDivisionError) as ze:
            _ = self.nepalinumber_integer_2 / 0

        self.assertEqual(str(ze.exception), "division by zero")

    def test_nepalinumber_throws_error_when_zero_nepalinumber_divides_other_numbers(
        self,
    ):
        with self.assertRaises(ZeroDivisionError) as ze:
            _ = 12 / self.nepalinumber_zero

        self.assertEqual(str(ze.exception), "division by zero")

    def test_nepalinumber_throws_error_when_divided_by_zero_nepalinumber(self):
        with self.assertRaises(ZeroDivisionError) as ze:
            _ = self.nepalinumber_integer_2 / self.nepalinumber_zero

        self.assertEqual(str(ze.exception), "division by zero")

    # floor division
    # nepalinumber positive integer division tests
    def test_nepalinumber_integer_is_floor_divisible_by_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_integer_20 // 2, self.nepalinumber_integer_10
        )

    def test_nepalinumber_integer_is_floor_divisible_by_positive_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 // 2.5, self.nepalinumber_integer_4
        )

    def test_nepalinumber_integer_is_floor_divisible_by_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_integer_20 // (-2), self.nepalinumber_negative_integer_10
        )

    def test_nepalinumber_integer_is_floor_divisible_by_negative_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 // (-2.5), self.nepalinumber_negative_integer_4
        )

    def test_nepalinumber_integer_is_floor_divisible_by_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_20 // self.nepalinumber_integer_10,
            self.nepalinumber_integer_2,
        )

    def test_nepalinumber_integer_is_floor_divisible_by_positive_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 / self.nepalinumber_float_2_5,
            self.nepalinumber_integer_4,
        )

    def test_nepalinumber_integer_is_floor_divisible_by_negative_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_20 // self.nepalinumber_negative_integer_10,
            self.nepalinumber_negative_integer_2,
        )

    def test_nepalinumber_integer_is_floor_divisible_by_negative_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 // self.nepalinumber_negative_float_2_5,
            self.nepalinumber_negative_integer_4,
        )

    # dividing other numbers by nepali number tests
    def test_positive_integer_is_floor_divisible_by_nepalinumber_integer(self):
        self.assertEqual(
            20 // self.nepalinumber_integer_2, self.nepalinumber_integer_10
        )

    def test_positive_float_is_floor_divisible_by_nepalinumber_integer(self):
        self.assertEqual(
            20.2 // self.nepalinumber_integer_2, self.nepalinumber_integer_10
        )

    def test_negative_integer_is_floor_divisible_by_nepalinumber_integer(self):
        self.assertEqual(
            -20 // self.nepalinumber_integer_10, self.nepalinumber_negative_integer_2
        )

    def test_negative_integer_is_floor_divisible_by_negative_nepalinumber_integer_to_give_positive_nepali_integer(
        self,
    ):
        self.assertEqual(
            (-20) // self.nepalinumber_negative_integer_10, self.nepalinumber_integer_2
        )

    def test_negative_float_is_floor_divisible_by_nepalinumber_integer(self):
        self.assertEqual(
            (-19.2) // self.nepalinumber_integer_2,
            self.nepalinumber_negative_integer_10,
        )

    def test_negative_float_is_floor_divisible_by_negative_nepalinumber_integer_to_give_positive_nepali_integer(
        self,
    ):
        self.assertEqual(
            (-20.2) // self.nepalinumber_negative_integer_2,
            self.nepalinumber_integer_10,
        )

    # nepalinumber negative integer division tests
    def test_negative_nepalinumber_integer_is_floor_divisible_by_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 // 5,
            self.nepalinumber_negative_integer_2,
        )

    def test_negative_nepalinumber_integer_is_floor_divisible_by_positive_float(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 // 2.5,
            self.nepalinumber_negative_integer_4,
        )

    def test_negative_nepalinumber_integer_is_floor_divisible_by_negative_integer_to_give_positive_nepali_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 // -5, self.nepalinumber_integer_2
        )

    def test_negative_nepalinumber_integer_is_floor_divisible_by_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 // (-5),
            self.nepalinumber_integer_2,
        )

    def test_negative_nepalinumber_integer_is_floor_divisible_by_negative_float(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 // (-2.5),
            self.nepalinumber_integer_4,
        )

    def test_negative_nepalinumber_integer_is_floor_divisible_by_negative_float_to_give_positive_nepali_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 // -2.5, self.nepalinumber_integer_4
        )

    def test_negative_nepalinumber_integer_is_floor_divisible_by_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_20 // self.nepalinumber_integer_10,
            self.nepalinumber_negative_integer_2,
        )

    def test_negative_nepalinumber_integer_is_floor_divisible_by_positive_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 // self.nepalinumber_float_2_5,
            self.nepalinumber_negative_integer_4,
        )

    def test_negative_nepalinumber_integer_is_floor_divisible_by_negative_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_20
            // self.nepalinumber_negative_integer_10,
            self.nepalinumber_integer_2,
        )

    def test_negative_nepalinumber_integer_is_floor_divisible_by_negative_nepali_number_integer_to_give_positive_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_20
            // self.nepalinumber_negative_integer_2,
            self.nepalinumber_integer_10,
        )

    def test_negative_nepalinumber_integer_is_floor_divisible_by_negative_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10
            // self.nepalinumber_negative_float_2_5,
            self.nepalinumber_integer_4,
        )

    # nepalinumber positive float division tests
    def test_nepalinumber_float_is_floor_divisible_by_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_float_20_2 // 2, self.nepalinumber_integer_10
        )

    def test_nepalinumber_float_is_floor_divisible_by_positive_float(self):
        self.assertEqual(
            self.nepalinumber_float_20_2 // 10.1, self.nepalinumber_integer_2
        )

    def test_nepalinumber_float_is_floor_divisible_by_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_float_20_2 // (-2), self.nepalinumber_negative_integer_11
        )

    def test_nepalinumber_float_is_floor_divisible_by_negative_float(self):
        self.assertEqual(
            self.nepalinumber_float_20_2 // (-10.1),
            self.nepalinumber_negative_integer_2,
        )

    def test_nepalinumber_float_is_floor_divisible_by_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_20_2 // self.nepalinumber_integer_2,
            self.nepalinumber_integer_10,
        )

    def test_nepalinumber_float_is_floor_divisible_by_positive_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_20_2 // self.nepalinumber_float_10_1,
            self.nepalinumber_integer_2,
        )

    def test_nepalinumber_float_is_floor_divisible_by_negative_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_20_2 // self.nepalinumber_negative_integer_2,
            self.nepalinumber_negative_integer_11,
        )

    def test_nepalinumber_float_is_floor_divisible_by_negative_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_20_2 // self.nepalinumber_negative_float_10_1,
            self.nepalinumber_negative_integer_2,
        )

    # nepalinumber negative float division tests
    def test_negative_nepalinumber_float_is_floor_divisible_by_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2 // 2,
            self.nepalinumber_negative_integer_11,
        )

    def test_negative_nepalinumber_float_is_floor_divisible_by_positive_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2 // 10.1,
            self.nepalinumber_negative_integer_2,
        )

    def test_negative_nepalinumber_float_is_floor_divisible_by_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2 // (-2),
            self.nepalinumber_integer_10,
        )

    def test_negative_nepalinumber_float_is_floor_divisible_by_negative_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2 // (-10.1),
            self.nepalinumber_integer_2,
        )

    def test_negative_nepalinumber_float_is_floor_divisible_by_positive_nepali_number_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2 // self.nepalinumber_integer_2,
            self.nepalinumber_negative_integer_11,
        )

    def test_negative_nepalinumber_float_is_floor_divisible_by_positive_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2 // self.nepalinumber_float_10_1,
            self.nepalinumber_negative_integer_2,
        )

    def test_negative_nepalinumber_float_is_floor_divisible_by_negative_nepali_number_integer_to_give_positive_nepali_number(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2
            // self.nepalinumber_negative_integer_2,
            self.nepalinumber_integer_10,
        )

    def test_negative_nepalinumber_float_is_floor_divisible_by_negative_nepali_number_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_20_2
            // self.nepalinumber_negative_float_10_1,
            self.nepalinumber_integer_2,
        )

    # division negative testing
    def test_nepalinumber_throws_error_when_floor_divided_by_objects_other_than_int_or_float(
        self,
    ):
        self.assertNotIsInstance(self.random_object, int)
        self.assertNotIsInstance(self.random_object, float)

        with self.assertRaises(TypeError) as te:
            _ = self.nepalinumber_integer_10 // self.random_object

        self.assertEqual(
            str(te.exception),
            "unsupported operand type(s) for //: 'nepalinumber' and 'type'",
        )

        with self.assertRaises(TypeError) as te:
            _ = self.nepalinumber_float_10_1 // self.random_object

        self.assertEqual(
            str(te.exception),
            "unsupported operand type(s) for //: 'nepalinumber' and 'type'",
        )

    def test_nepalinumber_throws_error_when_floor_divided_to_objects_other_than_int_or_float(
        self,
    ):
        self.assertNotIsInstance(self.random_object, int)
        self.assertNotIsInstance(self.random_object, float)

        with self.assertRaises(TypeError) as te:
            _ = self.random_object // self.nepalinumber_integer_10

        self.assertEqual(
            str(te.exception),
            "unsupported operand type(s) for //: 'type' and 'nepalinumber'",
        )

        with self.assertRaises(TypeError) as te:
            _ = self.random_object // self.nepalinumber_float_10_1

        self.assertEqual(
            str(te.exception),
            "unsupported operand type(s) for //: 'type' and 'nepalinumber'",
        )

    def test_nepalinumber_throws_error_when_floor_floor_divided_by_zero(self):
        with self.assertRaises(ZeroDivisionError) as ze:
            _ = self.nepalinumber_integer_2 // 0

        self.assertEqual(str(ze.exception), "integer division or modulo by zero")

        with self.assertRaises(ZeroDivisionError) as ze:
            _ = self.nepalinumber_float_10_1 // 0

        python_version = sys.version_info

        if python_version.major >= 3:
            if python_version.minor <= 8:
                self.assertEqual(str(ze.exception), "float divmod()")
            else:
                self.assertEqual(str(ze.exception), "float floor division by zero")

    def test_nepalinumber_throws_error_when_zero_nepalinumber_floor_divides_other_numbers(
        self,
    ):
        with self.assertRaises(ZeroDivisionError) as ze:
            _ = 12 // self.nepalinumber_zero

        self.assertEqual(str(ze.exception), "integer division or modulo by zero")

        with self.assertRaises(ZeroDivisionError) as ze:
            _ = 12.1 // self.nepalinumber_zero

        python_version = sys.version_info

        if python_version.major >= 3:
            if python_version.minor <= 8:
                self.assertEqual(str(ze.exception), "float divmod()")
            else:
                self.assertEqual(str(ze.exception), "float floor division by zero")

    def test_nepalinumber_throws_error_when_floor_divided_by_zero_nepalinumber(self):
        with self.assertRaises(ZeroDivisionError) as ze:
            _ = self.nepalinumber_integer_2 // self.nepalinumber_zero

        self.assertEqual(str(ze.exception), "integer division or modulo by zero")

        with self.assertRaises(ZeroDivisionError) as ze:
            _ = self.nepalinumber_float_10_1 // self.nepalinumber_zero

        python_version = sys.version_info

        if python_version.major >= 3:
            if python_version.minor <= 8:
                self.assertEqual(str(ze.exception), "float divmod()")
            else:
                self.assertEqual(str(ze.exception), "float floor division by zero")

    # mod
    # nepalinumber positive integer modulo tests
    def test_nepalinumber_integer_can_be_modulo_divided_by_positive_integer(self):
        self.assertEqual(self.nepalinumber_integer_10 % 4, self.nepalinumber_integer_2)

    def test_nepalinumber_integer_can_be_modulo_divided_by_positive_float(self):
        self.assertEqual(self.nepalinumber_integer_10 % 2.5, self.nepalinumber_zero)

    def test_nepalinumber_integer_can_be_modulo_divided_by_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_integer_10 % -4, self.nepalinumber_negative_integer_2
        )

    def test_nepalinumber_integer_can_be_modulo_divided_by_negative_float(self):
        self.assertEqual(
            self.nepalinumber_integer_10 % -2.25, self.nepalinumber_negative_float_1_25
        )

    def test_nepalinumber_integer_can_be_modulo_divided_by_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 % self.nepalinumber_integer_2,
            self.nepalinumber_zero,
        )

    def test_nepalinumber_integer_can_be_modulo_divided_by_positive_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 % self.nepalinumber_float_2_5,
            self.nepalinumber_zero,
        )

    def test_nepalinumber_integer_can_be_modulo_divided_by_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 % self.nepalinumber_negative_integer_20,
            self.nepalinumber_negative_integer_10,
        )

    def test_nepalinumber_integer_can_be_modulo_divided_by_negative_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_10 % self.nepalinumber_negative_float_15_5,
            self.nepalinumber_negative_float_5_5,
        )

    # nepalinumber other numbers modulo divided by positive nepalinumber tests
    def test_positive_integer_can_be_modulo_divided_by_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(4 % self.nepalinumber_integer_10, self.nepalinumber_integer_4)

    def test_positive_float_can_be_modulo_divided_by_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            15.5 % self.nepalinumber_integer_10, self.nepalinumber_float_5_5
        )

    def test_negative_integer_can_be_modulo_divided_by_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(-5 % self.nepalinumber_integer_10, self.nepalinumber_integer_5)

    def test_negative_float_can_be_modulo_divided_by_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            -20.5 % self.nepalinumber_integer_2, self.nepalinumber_float_1_5
        )

    def test_positive_nepalinumber_float_can_be_modulo_divided_by_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_10_1 % self.nepalinumber_integer_5,
            self.nepalinumber_float_0_0_9,
        )

    def test_negative_nepalinumber_integer_can_be_modulo_divided_by_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 % self.nepalinumber_integer_20,
            self.nepalinumber_integer_10,
        )

    def test_negative_nepalinumber_float_can_be_modulo_divided_by_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_15_5 % self.nepalinumber_integer_10,
            self.nepalinumber_float_4_5,
        )

    # nepalinumber negative integer modulo tests
    def test_negative_nepalinumber_integer_can_be_modulo_divided_by_positive_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 % 4, self.nepalinumber_integer_2
        )

    def test_negative_nepalinumber_integer_can_be_modulo_divided_by_positive_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 % 2.5, self.nepalinumber_zero
        )

    def test_negative_nepalinumber_integer_can_be_modulo_divided_by_negative_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 % -4,
            self.nepalinumber_negative_integer_2,
        )

    def test_negative_nepalinumber_integer_can_be_modulo_divided_by_negative_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 % -2.5, self.nepalinumber_zero
        )

    def test_negative_nepalinumber_integer_can_be_modulo_divided_by_positive_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10 % self.nepalinumber_float_2_5,
            self.nepalinumber_zero,
        )

    def test_negative_nepalinumber_integer_can_be_modulo_divided_by_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10
            % self.nepalinumber_negative_integer_20,
            self.nepalinumber_negative_integer_10,
        )

    def test_negative_nepalinumber_integer_can_be_modulo_divided_by_negative_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_10
            % self.nepalinumber_negative_float_15_5,
            self.nepalinumber_negative_integer_10,
        )

    # nepalinumber other numbers modulo divided by negative nepalinumber tests
    def test_positive_integer_can_be_modulo_divided_by_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            5 % self.nepalinumber_negative_integer_10,
            self.nepalinumber_negative_integer_5,
        )

    def test_positive_float_can_be_modulo_divided_by_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            15.5 % self.nepalinumber_negative_integer_10,
            self.nepalinumber_negative_float_4_5,
        )

    def test_negative_integer_can_be_modulo_divided_negative_nepalinumber_integer(self):
        self.assertEqual(
            -5 % self.nepalinumber_negative_integer_10,
            self.nepalinumber_negative_integer_5,
        )

    def test_negative_float_can_be_modulo_divided_by_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            -5.5 % self.nepalinumber_negative_integer_2,
            self.nepalinumber_negative_float_1_5,
        )

    def test_positive_nepalinumber_float_can_be_modulo_divided_by_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_4_5 % self.nepalinumber_negative_integer_10,
            self.nepalinumber_negative_float_5_5,
        )

    def test_negative_nepalinumber_float_can_be_modulo_divided_by_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_15_5
            % self.nepalinumber_negative_integer_10,
            self.nepalinumber_negative_float_5_5,
        )

    # nepalinumber positive float modulo tests
    def test_nepalinumber_float_can_be_modulo_divided_by_positive_integer(self):
        self.assertEqual(self.nepalinumber_float_4_5 % 2, self.nepalinumber_float_0_5)

    def test_nepalinumber_float_can_be_modulo_divided_by_positive_float(self):
        self.assertEqual(self.nepalinumber_float_4_5 % 2.5, self.nepalinumber_integer_2)

    def test_nepalinumber_float_can_be_modulo_divided_by_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_float_4_5 % -2, self.nepalinumber_negative_float_1_5
        )

    def test_nepalinumber_float_can_be_modulo_divided_by_negative_float(self):
        self.assertEqual(self.nepalinumber_float_4_5 % -2.25, self.nepalinumber_zero)

    def test_nepalinumber_float_can_be_modulo_divided_by_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_4_5 % self.nepalinumber_integer_2,
            self.nepalinumber_float_0_5,
        )

    def test_nepalinumber_float_can_be_modulo_divided_by_positive_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_4_5 % self.nepalinumber_float_2_25,
            self.nepalinumber_zero,
        )

    def test_nepalinumber_float_can_be_modulo_divided_by_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_4_5 % self.nepalinumber_negative_integer_20,
            self.nepalinumber_negative_float_15_5,
        )

    def test_nepalinumber_float_can_be_modulo_divided_by_negative_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_4_5 % self.nepalinumber_negative_float_15_5,
            self.nepalinumber_negative_integer_11,
        )

    # nepalinumber other numbers modulo divided by positive nepalinumber float tests
    def test_positive_nepalinumber_float_can_be_modulo_divided_by_positive_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_10_1 % self.nepalinumber_integer_5,
            self.nepalinumber_float_0_0_9,
        )

    # nepalinumber negative float modulo tests
    def test_negative_nepalinumber_float_can_be_modulo_divided_by_positive_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_15_5 % 4, self.nepalinumber_float_0_5
        )

    def test_negative_nepalinumber_float_can_be_modulo_divided_by_positive_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_15_5 % 2.5, self.nepalinumber_integer_2
        )

    def test_negative_nepalinumber_float_can_be_modulo_divided_by_negative_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_4_5 % -20,
            self.nepalinumber_negative_float_4_5,
        )

    def test_negative_nepalinumber_float_can_be_modulo_divided_by_negative_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_4_5 % -2.5,
            self.nepalinumber_negative_integer_2,
        )

    def test_negative_nepalinumber_float_can_be_modulo_divided_by_positive_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_15_5 % self.nepalinumber_float_2_5,
            self.nepalinumber_integer_2,
        )

    def test_negative_nepalinumber_float_can_be_modulo_divided_by_negative_nepalinumber_integer_2(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_4_5
            % self.nepalinumber_negative_integer_20,
            self.nepalinumber_negative_float_4_5,
        )

    # nepalinumber other integers modulo divided by negative nepalinumber float tests
    def test_positive_integer_can_be_modulo_divided_by_negative_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            5 % self.nepalinumber_negative_float_2_5, self.nepalinumber_zero
        )

    def test_positive_float_can_be_modulo_divided_by_negative_nepalinumber_float(self):
        self.assertEqual(
            15.5 % self.nepalinumber_negative_float_15_5, self.nepalinumber_zero
        )

    def test_negative_integer_can_be_modulo_divided_by_negative_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            -5 % self.nepalinumber_negative_float_2_5, self.nepalinumber_zero
        )

    def test_negative_float_can_be_modulo_divided_by_negative_nepalinumber_float(self):
        self.assertEqual(
            -5.5 % self.nepalinumber_negative_float_15_5,
            self.nepalinumber_negative_float_5_5,
        )

    def test_positive_nepalinumber_float_can_be_modulo_divided_by_negative_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_4_5 % self.nepalinumber_negative_float_15_5,
            self.nepalinumber_negative_integer_11,
        )

    # modulo negative_testing
    def test_nepalinumber_throws_error_when_object_other_than_int_or_float_modulo_divide(
        self,
    ):
        self.assertNotIsInstance(self.random_object, int)
        self.assertNotIsInstance(self.random_object, float)

        with self.assertRaises(TypeError) as te:
            _ = self.nepalinumber_integer_10 % self.random_object

        self.assertEqual(
            str(te.exception),
            "unsupported operand type(s) for %: 'nepalinumber' and 'type'",
        )

    def test_nepalinumber_throws_error_when_modulo_divided_with_objects_other_than_int_or_float(
        self,
    ):
        self.assertNotIsInstance(self.random_object, int)
        self.assertNotIsInstance(self.random_object, float)

        with self.assertRaises(TypeError) as te:
            _ = self.random_object % self.nepalinumber_integer_10

        self.assertEqual(
            str(te.exception),
            "unsupported operand type(s) for %: 'type' and 'nepalinumber'",
        )

    # divmod
    # divmod positive nepalinumber integer as dividend tests
    def test_positive_nepalinumber_integer_can_be_divmod_with_positive_integer(self):
        self.assertEqual(
            divmod(self.nepalinumber_integer_10, 2),
            (self.nepalinumber_integer_5, self.nepalinumber_zero),
        )

    def test_positive_nepalinumber_integer_can_be_divmod_with_positive_float(self):
        self.assertEqual(
            divmod(self.nepalinumber_integer_10, 2.5),
            (self.nepalinumber_integer_4, self.nepalinumber_zero),
        )

    def test_positive_nepalinumber_integer_can_be_divmod_with_negative_integer(self):
        self.assertEqual(
            divmod(self.nepalinumber_integer_10, -4),
            (
                self.nepalinumber_negative_integer_3,
                self.nepalinumber_negative_integer_2,
            ),
        )

    def test_positive_nepalinumber_integer_can_be_divmod_with_negative_float(self):
        self.assertEqual(
            divmod(self.nepalinumber_integer_10, -5.5),
            (
                self.nepalinumber_negative_integer_2,
                self.nepalinumber_negative_integer_1,
            ),
        )

    def test_positive_nepalinumber_integer_can_be_divmod_with_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            divmod(self.nepalinumber_integer_10, self.nepalinumber_integer_2),
            (self.nepalinumber_integer_5, self.nepalinumber_zero),
        )

    def test_positive_nepalinumber_integer_can_be_divmod_with_positive_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            divmod(self.nepalinumber_integer_10, self.nepalinumber_float_2_5),
            (self.nepalinumber_integer_4, self.nepalinumber_zero),
        )

    def test_positive_nepalinumber_integer_can_be_divmod_with_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            divmod(self.nepalinumber_integer_10, self.nepalinumber_negative_integer_4),
            (
                self.nepalinumber_negative_integer_3,
                self.nepalinumber_negative_integer_2,
            ),
        )

    def test_positive_nepalinumber_integer_can_be_divmod_with_negative_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            divmod(self.nepalinumber_integer_10, self.nepalinumber_negative_float_5_5),
            (
                self.nepalinumber_negative_integer_2,
                self.nepalinumber_negative_integer_1,
            ),
        )

    # divmod positive nepalinumber integer as divisor tests
    def test_positive_integer_can_be_divmod_with_positive_nepalinumber_integer(self):
        self.assertEqual(
            divmod(10, self.nepalinumber_integer_2),
            (self.nepalinumber_integer_5, self.nepalinumber_zero),
        )

    def test_positive_float_can_be_divmod_with_positive_nepalinumber_integer(self):
        self.assertEqual(
            divmod(15.5, self.nepalinumber_integer_10),
            (self.nepalinumber_integer_1, self.nepalinumber_float_5_5),
        )

    def test_negative_integer_can_be_divmod_with_positive_nepalinumber_integer(self):
        self.assertEqual(
            divmod(-10, self.nepalinumber_integer_10),
            (self.nepalinumber_negative_integer_1, self.nepalinumber_zero),
        )

    def test_negative_float_can_be_divmod_with_positive_nepalinumber_integer(self):
        self.assertEqual(
            divmod(-15.5, self.nepalinumber_integer_5),
            (self.nepalinumber_negative_integer_4, self.nepalinumber_float_4_5),
        )

    def test_positive_nepalinumber_float_can_be_divmod_with_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            divmod(self.nepalinumber_float_15_5, self.nepalinumber_integer_10),
            (self.nepalinumber_integer_1, self.nepalinumber_float_5_5),
        )

    def test_negative_nepalinumber_integer_can_be_divmod_with_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            divmod(self.nepalinumber_negative_integer_10, self.nepalinumber_integer_10),
            (self.nepalinumber_negative_integer_1, self.nepalinumber_zero),
        )

    def test_negative_nepalinumber_float_can_be_divmod_with_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            divmod(self.nepalinumber_negative_float_15_5, self.nepalinumber_integer_5),
            (self.nepalinumber_negative_integer_4, self.nepalinumber_float_4_5),
        )

    # divmod positive nepalinumber float as dividend tests
    def test_positive_nepalinumber_float_can_be_divmod_with_positive_integer(self):
        self.assertEqual(
            divmod(self.nepalinumber_float_4_5, 2),
            (self.nepalinumber_integer_2, self.nepalinumber_float_0_5),
        )

    def test_positive_nepalinumber_float_can_be_divmod_with_positive_float(self):
        self.assertEqual(
            divmod(self.nepalinumber_float_4_5, 2.5),
            (self.nepalinumber_integer_1, self.nepalinumber_integer_2),
        )

    def test_positive_nepalinumber_float_can_be_divmod_with_negative_integer(self):
        self.assertEqual(
            divmod(self.nepalinumber_float_4_5, -3),
            (
                self.nepalinumber_negative_integer_2,
                self.nepalinumber_negative_float_1_5,
            ),
        )

    def test_positive_nepalinumber_float_can_be_divmod_with_negative_float(self):
        self.assertEqual(
            divmod(self.nepalinumber_float_4_5, -2.25),
            (self.nepalinumber_negative_integer_2, self.nepalinumber_zero),
        )

    def test_positive_nepalinumber_float_can_be_divmod_with_positive_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            divmod(self.nepalinumber_float_4_5, self.nepalinumber_float_4_5),
            (self.nepalinumber_integer_1, self.nepalinumber_zero),
        )

    def test_positive_nepalinumber_float_can_be_divmod_with_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            divmod(self.nepalinumber_float_4_5, -2),
            (
                self.nepalinumber_negative_integer_3,
                self.nepalinumber_negative_float_1_5,
            ),
        )

    def test_positive_nepalinumber_float_can_be_divmod_with_negative_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            divmod(self.nepalinumber_float_4_5, -2.5),
            (
                self.nepalinumber_negative_integer_2,
                self.nepalinumber_negative_float_0_5,
            ),
        )

    # divmod positive nepalinumber float as divisor tests
    def test_positive_integer_can_be_divmod_with_positive_nepalinumber_float(self):
        self.assertEqual(
            divmod(10, self.nepalinumber_float_2_5),
            (self.nepalinumber_integer_4, self.nepalinumber_zero),
        )

    def test_positive_float_can_be_divmod_with_positive_nepalinumber_float(self):
        self.assertEqual(
            divmod(4.5, self.nepalinumber_float_2_25),
            (self.nepalinumber_integer_2, self.nepalinumber_zero),
        )

    def test_negative_integer_can_be_divmod_with_positive_nepalinumber_float(self):
        self.assertEqual(
            divmod(-10, self.nepalinumber_float_2_25),
            (self.nepalinumber_negative_integer_5, self.nepalinumber_float_1_25),
        )

    def test_negative_float_can_be_divmod_with_positive_nepalinumber_float(self):
        self.assertEqual(
            divmod(-15.5, self.nepalinumber_float_1_5),
            (self.nepalinumber_negative_integer_11, self.nepalinumber_integer_1),
        )

    def test_negative_nepalinumber_integer_can_be_divmod_with_positive_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            divmod(self.nepalinumber_negative_integer_10, self.nepalinumber_float_2_5),
            (self.nepalinumber_negative_integer_4, self.nepalinumber_zero),
        )

    def test_negative_nepalinumber_float_can_be_divmod_with_positive_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            divmod(self.nepalinumber_negative_float_20_2, self.nepalinumber_float_10_1),
            (self.nepalinumber_negative_integer_2, self.nepalinumber_zero),
        )

    # divmod negative nepalinumber integer as dividend tests
    def test_negative_nepalinumber_integer_can_be_divmod_with_positive_integer(self):
        self.assertEqual(
            divmod(self.nepalinumber_negative_integer_10, 2),
            (self.nepalinumber_negative_integer_5, self.nepalinumber_zero),
        )

    def test_negative_nepalinumber_integer_can_be_divmod_with_positive_float(self):
        self.assertEqual(
            divmod(self.nepalinumber_negative_integer_10, 2.5),
            (self.nepalinumber_negative_integer_4, self.nepalinumber_zero),
        )

    def test_negative_nepalinumber_integer_can_be_divmod_with_negative_integer(self):
        self.assertEqual(
            divmod(self.nepalinumber_negative_integer_10, -4),
            (self.nepalinumber_integer_2, self.nepalinumber_negative_integer_2),
        )

    def test_negative_nepalinumber_integer_can_be_divmod_with_negative_float(self):
        self.assertEqual(
            divmod(self.nepalinumber_negative_integer_10, -5.5),
            (self.nepalinumber_integer_1, self.nepalinumber_negative_float_4_5),
        )

    def test_negative_nepalinumber_integer_can_be_divmod_with_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            divmod(
                self.nepalinumber_negative_integer_10,
                self.nepalinumber_negative_integer_4,
            ),
            (self.nepalinumber_integer_2, self.nepalinumber_negative_integer_2),
        )

    def test_negative_nepalinumber_integer_can_be_divmod_with_negative_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            divmod(
                self.nepalinumber_negative_integer_10,
                self.nepalinumber_negative_float_5_5,
            ),
            (self.nepalinumber_integer_1, self.nepalinumber_negative_float_4_5),
        )

    # divmod negative nepalinumber integer as divisor tests
    def test_positive_integer_can_be_divmod_with_negative_nepalinumber_integer(self):
        self.assertEqual(
            divmod(10, self.nepalinumber_negative_integer_2),
            (self.nepalinumber_negative_integer_5, self.nepalinumber_zero),
        )

    def test_positive_float_can_be_divmod_with_negative_nepalinumber_integer(self):
        self.assertEqual(
            divmod(15.5, self.nepalinumber_negative_integer_10),
            (
                self.nepalinumber_negative_integer_2,
                self.nepalinumber_negative_float_4_5,
            ),
        )

    def test_negative_integer_can_be_divmod_with_negative_nepalinumber_integer(self):
        self.assertEqual(
            divmod(-10, self.nepalinumber_negative_integer_10),
            (self.nepalinumber_integer_1, self.nepalinumber_zero),
        )

    def test_negative_float_can_be_divmod_with_negative_nepalinumber_integer(self):
        self.assertEqual(
            divmod(-4.5, self.nepalinumber_negative_integer_5),
            (self.nepalinumber_zero, self.nepalinumber_negative_float_4_5),
        )

    def test_negative_nepalinumber_float_can_be_divmod_with_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            divmod(
                self.nepalinumber_negative_float_4_5,
                self.nepalinumber_negative_integer_2,
            ),
            (self.nepalinumber_integer_2, self.nepalinumber_negative_float_0_5),
        )

    # divmod negative nepalinumber float as dividend tests
    def test_negative_nepalinumber_float_can_be_divmod_with_positive_integer(self):
        self.assertEqual(
            divmod(self.nepalinumber_negative_float_4_5, 2),
            (self.nepalinumber_negative_integer_3, self.nepalinumber_float_1_5),
        )

    def test_negative_nepalinumber_float_can_be_divmod_with_positive_float(self):
        self.assertEqual(
            divmod(self.nepalinumber_negative_float_4_5, 2.5),
            (self.nepalinumber_negative_integer_2, self.nepalinumber_float_0_5),
        )

    def test_negative_nepalinumber_float_can_be_divmod_with_negative_integer(self):
        self.assertEqual(
            divmod(self.nepalinumber_negative_float_4_5, -3),
            (self.nepalinumber_integer_1, self.nepalinumber_negative_float_1_5),
        )

    def test_negative_nepalinumber_float_can_be_divmod_with_negative_float(self):
        self.assertEqual(
            divmod(self.nepalinumber_negative_float_4_5, -2.25),
            (self.nepalinumber_integer_2, self.nepalinumber_zero),
        )

    def test_negative_nepalinumber_float_can_be_divmod_with_negative_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            divmod(self.nepalinumber_negative_float_4_5, -2.5),
            (self.nepalinumber_integer_1, self.nepalinumber_negative_integer_2),
        )

    # divmod negative nepalinumber float as divisor tests
    def test_positive_integer_can_be_divmod_with_negative_nepalinumber_float(self):
        self.assertEqual(
            divmod(10, self.nepalinumber_negative_float_2_5),
            (self.nepalinumber_negative_integer_4, self.nepalinumber_zero),
        )

    def test_positive_float_can_be_divmod_with_negative_nepalinumber_float(self):
        self.assertEqual(
            divmod(4.5, self.nepalinumber_negative_float_2_25),
            (self.nepalinumber_negative_integer_2, self.nepalinumber_zero),
        )

    def test_negative_integer_can_be_divmod_with_negative_nepalinumber_float(self):
        self.assertEqual(
            divmod(-10, self.nepalinumber_negative_float_2_25),
            (self.nepalinumber_integer_4, self.nepalinumber_negative_integer_1),
        )

    def test_negative_float_can_be_divmod_with_negative_nepalinumber_float(self):
        self.assertEqual(
            divmod(-15.5, self.nepalinumber_negative_float_1_5),
            (self.nepalinumber_integer_10, self.nepalinumber_negative_float_0_5),
        )

    # pow
    # pow positive nepalinumber integer as base
    def test_positive_nepalinumber_integer_can_be_powered_to_positive_integer(self):
        self.assertEqual(self.nepalinumber_integer_2**2, self.nepalinumber_integer_4)

    def test_positive_nepalinumber_integer_can_be_powered_to_positive_float(self):
        self.assertEqual(
            self.nepalinumber_integer_2**2.5, number.nepalinumber(2**2.5)
        )

    def test_positive_nepalinumber_integer_can_be_powered_to_negative_integer(self):
        self.assertEqual(self.nepalinumber_integer_2**-1, self.nepalinumber_float_0_5)

    def test_positive_nepalinumber_integer_can_be_powered_to_negative_float(self):
        self.assertEqual(
            self.nepalinumber_integer_2**-1.5, number.nepalinumber(2**-1.5)
        )

    def test_positive_nepalinumber_integer_can_be_powered_to_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_2**self.nepalinumber_integer_2,
            self.nepalinumber_integer_4,
        )

    def test_positive_nepalinumber_integer_can_be_powered_to_positive_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_2**self.nepalinumber_float_2_5,
            number.nepalinumber(2**2.5),
        )

    def test_positive_nepalinumber_integer_can_be_powered_to_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_2**self.nepalinumber_negative_integer_1,
            self.nepalinumber_float_0_5,
        )

    def test_positive_nepalinumber_integer_can_be_powered_to_negative_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_integer_2**self.nepalinumber_negative_float_1_5,
            number.nepalinumber(2**-1.5),
        )

    # pow positive nepalinumber integer as exponent
    def test_positive_integer_can_be_powered_to_positive_nepalinumber_integer(self):
        self.assertEqual(2**self.nepalinumber_integer_2, self.nepalinumber_integer_4)

    def test_positive_float_can_be_powered_to_positive_nepalinumber_integer(self):
        self.assertEqual(
            2.5**self.nepalinumber_integer_2, number.nepalinumber(2.5**2)
        )

    def test_negative_integer_can_be_powered_to_positive_nepalinumber_integer(self):
        self.assertEqual(
            (-1) ** self.nepalinumber_integer_2, self.nepalinumber_integer_1
        )

    def test_negative_float_can_be_powered_to_positive_nepalinumber_integer(self):
        self.assertEqual(
            (-1.5) ** self.nepalinumber_integer_2, number.nepalinumber((-1.5) ** 2)
        )

    def test_positive_nepalinumber_float_can_be_powered_to_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_2_5**self.nepalinumber_integer_2,
            number.nepalinumber(2.5**2),
        )

    def test_negative_nepalinumber_integer_can_be_powered_to_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_1**self.nepalinumber_integer_2,
            self.nepalinumber_integer_1,
        )

    def test_negative_nepalinumber_float_can_be_powered_to_positive_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_1_5**self.nepalinumber_integer_2,
            number.nepalinumber((-1.5) ** 2),
        )

    # pow positive nepalinumber float as base
    def test_positive_nepalinumber_float_can_be_powered_to_positive_integer(self):
        self.assertEqual(self.nepalinumber_float_1_5**2, self.nepalinumber_float_2_25)

    def test_positive_nepalinumber_float_can_be_powered_to_positive_float(self):
        self.assertEqual(
            self.nepalinumber_float_1_5**1.5, number.nepalinumber(1.5**1.5)
        )

    def test_positive_nepalinumber_float_can_be_powered_to_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_float_1_5**-2, number.nepalinumber(1.5**-2)
        )

    def test_positive_nepalinumber_float_can_be_powered_to_negative_float(self):
        self.assertEqual(
            self.nepalinumber_float_1_5**-1.5, number.nepalinumber(1.5**-1.5)
        )

    def test_positive_nepalinumber_float_can_be_powered_to_positive_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_1_5**self.nepalinumber_float_1_5,
            number.nepalinumber(1.5**1.5),
        )

    def test_positive_nepalinumber_float_can_be_powered_to_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_1_5**self.nepalinumber_negative_integer_2,
            number.nepalinumber(1.5**-2),
        )

    def test_positive_nepalinumber_float_can_be_powered_to_negative_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_float_1_5**self.nepalinumber_negative_float_1_5,
            number.nepalinumber(1.5**-1.5),
        )

    # pow positive nepalinumber float as exponent
    def test_positive_integer_can_be_powered_to_positive_nepalinumber_float(self):
        self.assertEqual(
            2**self.nepalinumber_float_1_5, number.nepalinumber(2**1.5)
        )

    def test_positive_float_can_be_powered_to_positive_nepalinumber_float(self):
        self.assertEqual(
            1.5**self.nepalinumber_float_1_5, number.nepalinumber(1.5**1.5)
        )

    def test_negative_integer_can_be_powered_to_positive_nepalinumber_float(self):
        self.assertEqual((-2) ** self.nepalinumber_float_1_5, (-2) ** 1.5)

    def test_negative_float_can_be_powered_to_positive_nepalinumber_float(self):
        self.assertEqual((-1.5) ** self.nepalinumber_float_1_5, (-1.5) ** 1.5)

    def test_negative_nepalinumber_integer_can_be_powered_to_positive_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_2**self.nepalinumber_float_1_5,
            (-2) ** 1.5,
        )

    def test_negative_nepalinumber_float_can_be_powered_to_positive_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_1_5**self.nepalinumber_float_1_5,
            (-1.5) ** 1.5,
        )

    # pow negative nepalinumber integer as base
    def test_negative_nepalinumber_integer_can_be_powered_to_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_2**2, self.nepalinumber_integer_4
        )

    def test_negative_nepalinumber_integer_can_be_powered_to_positive_float(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_2**2.5,
            (-2) ** 2.5,
        )

    def test_negative_nepalinumber_integer_can_be_powered_to_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_2**-1,
            self.nepalinumber_negative_float_0_5,
        )

    def test_negative_nepalinumber_integer_can_be_powered_to_negative_float(self):
        self.assertEqual(
            self.nepalinumber_negative_integer_2**-1.5,
            (-2) ** -1.5,
        )

    def test_negative_nepalinumber_integer_can_be_powered_to_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_2
            ** self.nepalinumber_negative_integer_1,
            self.nepalinumber_negative_float_0_5,
        )

    def test_negative_nepalinumber_integer_can_be_powered_to_negative_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_integer_2
            ** self.nepalinumber_negative_float_1_5,
            (-2) ** -1.5,
        )

    # pow negative nepalinumber integer as exponent
    def test_positive_integer_can_be_powered_to_negative_nepalinumber_integer(self):
        self.assertEqual(
            2**self.nepalinumber_negative_integer_2, number.nepalinumber(2**-2)
        )

    def test_positive_float_can_be_powered_to_negative_nepalinumber_integer(self):
        self.assertEqual(
            2.5**self.nepalinumber_negative_integer_2, number.nepalinumber(2.5**-2)
        )

    def test_negative_integer_can_be_powered_to_negative_nepalinumber_integer(self):
        self.assertEqual(
            (-1) ** self.nepalinumber_negative_integer_2, self.nepalinumber_integer_1
        )

    def test_negative_float_can_be_powered_to_negative_nepalinumber_integer(self):
        self.assertEqual(
            (-1.5) ** self.nepalinumber_negative_integer_2,
            number.nepalinumber((-1.5) ** (-2)),
        )

    def test_negative_nepalinumber_float_can_be_powered_to_negative_nepalinumber_integer(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_1_5
            ** self.nepalinumber_negative_integer_2,
            number.nepalinumber((-1.5) ** (-2)),
        )

    # pow negative nepalinumber float as base
    def test_negative_nepalinumber_float_can_be_powered_to_positive_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_float_1_5**2, self.nepalinumber_float_2_25
        )

    def test_negative_nepalinumber_float_can_be_powered_to_positive_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_1_5**1.5,
            (-1.5) ** 1.5,
        )

    def test_negative_nepalinumber_float_can_be_powered_to_negative_integer(self):
        self.assertEqual(
            self.nepalinumber_negative_float_1_5**-2,
            number.nepalinumber((-1.5) ** -2),
        )

    def test_negative_nepalinumber_float_can_be_powered_to_negative_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_1_5**-1.5,
            (-1.5) ** -1.5,
        )

    def test_negative_nepalinumber_float_can_be_powered_to_negative_nepalinumber_float(
        self,
    ):
        self.assertEqual(
            self.nepalinumber_negative_float_1_5
            ** self.nepalinumber_negative_float_1_5,
            (-1.5) ** -1.5,
        )

    # pow negative nepalinumber float as exponent
    def test_positive_integer_can_be_powered_to_negative_nepalinumber_float(self):
        self.assertEqual(
            2**self.nepalinumber_negative_float_1_5, number.nepalinumber(2**-1.5)
        )

    def test_positive_float_can_be_powered_to_negative_nepalinumber_float(self):
        self.assertEqual(
            1.5**self.nepalinumber_negative_float_1_5,
            number.nepalinumber(1.5**-1.5),
        )

    def test_negative_integer_can_be_powered_to_negative_nepalinumber_float(self):
        self.assertEqual(
            (-2) ** self.nepalinumber_negative_float_1_5,
            (-2) ** -1.5,
        )

    def test_negative_float_can_be_powered_to_negative_nepalinumber_float(self):
        self.assertEqual(
            (-1.5) ** self.nepalinumber_negative_float_1_5,
            (-1.5) ** -1.5,
        )


class TestNepaliNumberParse(unittest.TestCase):
    def test_nepalinumber_parse_int(self):
        num = nepalinumber(13)
        self.assertEqual(num.value, 13)
        self.assertEqual(type(num.value), int)

    def test_nepalinumber_parse_int_negative(self):
        num = nepalinumber(-13)
        self.assertEqual(num.value, -13)
        self.assertEqual(type(num.value), int)

    def test_nepalinumber_parse_float(self):
        num = nepalinumber(13.07)
        self.assertEqual(num.value, 13.07)
        self.assertEqual(type(num.value), float)

    def test_nepalinumber_parse_float_negative(self):
        num = nepalinumber(-13.07)
        self.assertEqual(num.value, -13.07)
        self.assertEqual(type(num.value), float)

    def test_nepalinumber_parse_str_int(self):
        num = nepalinumber("13")
        self.assertEqual(num.value, 13)
        self.assertEqual(type(num.value), int)

    def test_nepalinumber_parse_str_int_negative(self):
        num = nepalinumber("-13")
        self.assertEqual(num.value, -13)
        self.assertEqual(type(num.value), int)

    def test_nepalinumber_parse_str_float(self):
        num = nepalinumber("13.07")
        self.assertEqual(num.value, 13.07)
        self.assertEqual(type(num.value), float)

    def test_nepalinumber_parse_str_float_negative(self):
        num = nepalinumber("-13.07")
        self.assertEqual(num.value, -13.07)
        self.assertEqual(type(num.value), float)

    def test_nepalinumber_parse_str_float_starting_dot(self):
        num = nepalinumber(".07")
        self.assertEqual(num.value, 0.07)
        self.assertEqual(type(num.value), float)

    def test_nepalinumber_parse_str_float_starting_dot_and_negative(self):
        num = nepalinumber("-.07")
        self.assertEqual(num.value, -0.07)
        self.assertEqual(type(num.value), float)

    def test_nepalinumber_parse_str_nepali_int(self):
        num = nepalinumber("१३")
        self.assertEqual(num.value, 13)
        self.assertEqual(type(num.value), int)

    def test_nepalinumber_parse_str_nepali_int_negative(self):
        num = nepalinumber("-१३")
        self.assertEqual(num.value, -13)
        self.assertEqual(type(num.value), int)

    def test_nepalinumber_parse_str_nepali_float(self):
        num = nepalinumber("१३.०७")
        self.assertEqual(num.value, 13.07)
        self.assertEqual(type(num.value), float)

    def test_nepalinumber_parse_str_nepali_float_negative(self):
        num = nepalinumber("-१३.०७")
        self.assertEqual(num.value, -13.07)
        self.assertEqual(type(num.value), float)

    def test_nepalinumber_raises_exception_for_chars_english(self):
        with self.assertRaises(ValueError) as ex:
            nepalinumber("invalid")
        self.assertEqual(
            str(ex.exception), "could not convert str to nepalinumber: 'invalid'"
        )

    def test_nepalinumber_raises_exception_for_chars_nepali(self):
        with self.assertRaises(ValueError):
            nepalinumber("आइतबार")

    def test_nepalinumber_raises_exception_for_two_dots(self):
        with self.assertRaises(ValueError):
            nepalinumber("10..5")

    def test_nepalinumber_raises_exception_for_nepali_aplha(self):
        with self.assertRaises(ValueError):
            nepalinumber("10..5")

    def test_nepalinumber_raises_exception_for_special_chars(self):
        with self.assertRaises(ValueError):
            nepalinumber("!@#$%^&*()-+")

    def test_nepalinumber_raise_exception_for_number_with_spaces(self):
        with self.assertRaises(ValueError):
            nepalinumber("10 20")

    def test_nepalinumber_raises_exception_on_complex_number(self):
        with self.assertRaises(TypeError) as ex:
            nepalinumber(3 + 2j)
        self.assertEqual(
            str(ex.exception), "could not convert complex to nepalinumber: '(3+2j)'"
        )

    # Object parsing
    def test_nepalinumber_parse_object_with_int(self):
        class _test:
            def __int__(self):
                return 13

        num = nepalinumber(_test())
        self.assertEqual(num.value, int(_test()))
        self.assertEqual(type(num.value), int)

    def test_nepalinumber_parse_object_with_float(self):
        class _test:
            def __float__(self):
                return 13.07

        num = nepalinumber(_test())
        self.assertEqual(num.value, float(_test()))
        self.assertEqual(type(num.value), float)

    def test_nepalinumber_parse_object_with_float_and_int(self):
        class _test:
            def __int__(self):
                return 13

            def __float__(self):
                return 13.07

        num = nepalinumber(_test())
        self.assertNotEqual(num.value, int(_test()))
        self.assertEqual(num.value, float(_test()))
        self.assertEqual(type(num.value), float)

    def test_nepalinumber_parse_object_with_valid_number_str_int(self):
        class _test:
            def __str__(self):
                return "13"

        num = nepalinumber(_test())
        self.assertEqual(num.value, int(str(_test())))
        self.assertEqual(type(num.value), int)

    def test_nepalinumber_parse_object_with_valid_number_str_float(self):
        class _test:
            def __str__(self):
                return "13.07"

        num = nepalinumber(_test())
        self.assertEqual(num.value, float(str(_test())))
        self.assertEqual(type(num.value), float)

    def test_nepalinumber_raises_exception_for_invalid_object(self):
        class _test:
            def __str__(self):
                return "13-03"

        with self.assertRaises(TypeError) as ex:
            nepalinumber(_test())
        self.assertEqual(
            str(ex.exception), "could not convert _test to nepalinumber: '13-03'"
        )

    # parsing inheritance with float and int
    def test_nepalinumber_parses_inherited_int(self):
        class _test(int):
            pass

        num = nepalinumber(_test(13))
        self.assertEqual(num.value, 13)
        self.assertEqual(type(num.value), int)

    def test_nepalinumber_parses_inherited_float(self):
        class _test(float):
            pass

        num = nepalinumber(_test(13.07))
        self.assertEqual(num.value, 13.07)
        self.assertEqual(type(num.value), float)


class TestNepaliNumberMethods(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # positive integers
        cls.nepalinumber_integer_999999 = number.nepalinumber(999999)
        cls.nepalinumber_integer_1 = number.nepalinumber(1)
        cls.nepalinumber_integer_0 = number.nepalinumber(0)
        # positive floats
        cls.nepalinumber_float_999999_999999 = number.nepalinumber(999999.999999)
        cls.nepalinumber_float_1_25 = number.nepalinumber(1.25)
        cls.nepalinumber_float_0_05 = number.nepalinumber(0.05)
        cls.nepalinumber_float_0_0 = number.nepalinumber(0.0)
        # negative numbers
        cls.nepalinumber_negative_integer_999999 = number.nepalinumber(-999999)
        cls.nepalinumber_negative_integer_1 = number.nepalinumber(-1)
        # negative floats
        cls.nepalinumber_negative_float_999999_999999 = number.nepalinumber(
            -999999.999999
        )
        cls.nepalinumber_negative_float_1_25 = number.nepalinumber(-1.25)
        cls.nepalinumber_negative_float_0_05 = number.nepalinumber(-0.05)

    # tests for str_ne methods
    def test_nepalinumber_str_ne_for_small_positive_integer(self):
        self.assertEqual(self.nepalinumber_integer_1.str_ne(), "१")

    def test_nepalinumber_str_ne_for_large_positive_integer(self):
        self.assertEqual(self.nepalinumber_integer_999999.str_ne(), "९९९९९९")

    def test_nepalinumber_str_ne_for_small_negative_integer(self):
        self.assertEqual(self.nepalinumber_negative_integer_1.str_ne(), "-१")

    def test_nepalinumber_str_ne_for_large_negative_integer(self):
        self.assertEqual(self.nepalinumber_negative_integer_999999.str_ne(), "-९९९९९९")

    def test_nepalinumber_str_ne_for_small_positive_float(self):
        self.assertEqual(self.nepalinumber_float_1_25.str_ne(), "१.२५")
        self.assertEqual(self.nepalinumber_float_0_05.str_ne(), "०.०५")

    def test_nepalinumber_str_ne_for_large_positive_float(self):
        self.assertEqual(
            self.nepalinumber_float_999999_999999.str_ne(), "९९९९९९.९९९९९९"
        )

    def test_nepalinumber_str_ne_for_small_negative_float(self):
        self.assertEqual(self.nepalinumber_negative_float_1_25.str_ne(), "-१.२५")
        self.assertEqual(self.nepalinumber_negative_float_0_05.str_ne(), "-०.०५")

    def test_nepalinumber_str_ne_for_large_negative_float(self):
        self.assertEqual(
            self.nepalinumber_negative_float_999999_999999.str_ne(), "-९९९९९९.९९९९९९"
        )

    def test_nepalinumber_str_ne_for_zero(self):
        self.assertEqual(self.nepalinumber_integer_0.str_ne(), "०")
        self.assertEqual(self.nepalinumber_float_0_0.str_ne(), "०.०")
